# Generated from CnsMRParser.g4 by ANTLR 4.9
# encoding: utf-8
from antlr4 import *
from io import StringIO
import sys
if sys.version_info[1] > 5:
	from typing import TextIO
else:
	from typing.io import TextIO


def serializedATN():
    with StringIO() as buf:
        buf.write("\3\u608b\ua72a\u8133\ub9ed\u417c\u3be7\u7786\u5964\3\u00a1")
        buf.write("\u04fa\4\2\t\2\4\3\t\3\4\4\t\4\4\5\t\5\4\6\t\6\4\7\t\7")
        buf.write("\4\b\t\b\4\t\t\t\4\n\t\n\4\13\t\13\4\f\t\f\4\r\t\r\4\16")
        buf.write("\t\16\4\17\t\17\4\20\t\20\4\21\t\21\4\22\t\22\4\23\t\23")
        buf.write("\4\24\t\24\4\25\t\25\4\26\t\26\4\27\t\27\4\30\t\30\4\31")
        buf.write("\t\31\4\32\t\32\4\33\t\33\4\34\t\34\4\35\t\35\4\36\t\36")
        buf.write("\4\37\t\37\4 \t \4!\t!\4\"\t\"\4#\t#\4$\t$\4%\t%\4&\t")
        buf.write("&\4\'\t\'\4(\t(\4)\t)\4*\t*\4+\t+\4,\t,\4-\t-\4.\t.\4")
        buf.write("/\t/\4\60\t\60\4\61\t\61\4\62\t\62\4\63\t\63\4\64\t\64")
        buf.write("\4\65\t\65\4\66\t\66\4\67\t\67\3\2\3\2\3\2\3\2\3\2\3\2")
        buf.write("\3\2\3\2\3\2\3\2\3\2\3\2\3\2\7\2|\n\2\f\2\16\2\177\13")
        buf.write("\2\3\2\7\2\u0082\n\2\f\2\16\2\u0085\13\2\3\2\7\2\u0088")
        buf.write("\n\2\f\2\16\2\u008b\13\2\3\2\7\2\u008e\n\2\f\2\16\2\u0091")
        buf.write("\13\2\3\2\7\2\u0094\n\2\f\2\16\2\u0097\13\2\3\2\7\2\u009a")
        buf.write("\n\2\f\2\16\2\u009d\13\2\3\2\3\2\3\3\3\3\7\3\u00a3\n\3")
        buf.write("\f\3\16\3\u00a6\13\3\3\3\3\3\3\4\3\4\3\4\7\4\u00ad\n\4")
        buf.write("\f\4\16\4\u00b0\13\4\3\4\3\4\3\5\3\5\3\5\7\5\u00b7\n\5")
        buf.write("\f\5\16\5\u00ba\13\5\3\5\3\5\3\6\3\6\3\6\7\6\u00c1\n\6")
        buf.write("\f\6\16\6\u00c4\13\6\3\6\3\6\3\7\3\7\7\7\u00ca\n\7\f\7")
        buf.write("\16\7\u00cd\13\7\3\7\3\7\3\b\3\b\7\b\u00d3\n\b\f\b\16")
        buf.write("\b\u00d6\13\b\3\b\3\b\3\t\3\t\7\t\u00dc\n\t\f\t\16\t\u00df")
        buf.write("\13\t\3\t\3\t\3\n\3\n\7\n\u00e5\n\n\f\n\16\n\u00e8\13")
        buf.write("\n\3\n\3\n\3\13\3\13\7\13\u00ee\n\13\f\13\16\13\u00f1")
        buf.write("\13\13\3\13\3\13\3\f\3\f\7\f\u00f7\n\f\f\f\16\f\u00fa")
        buf.write("\13\f\3\f\3\f\3\r\3\r\7\r\u0100\n\r\f\r\16\r\u0103\13")
        buf.write("\r\3\r\3\r\3\16\3\16\7\16\u0109\n\16\f\16\16\16\u010c")
        buf.write("\13\16\3\16\3\16\3\17\3\17\5\17\u0112\n\17\3\17\3\17\3")
        buf.write("\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\5\17\u0121\n\17\3\17\3\17\3\17\5\17\u0126\n\17\3\17\3")
        buf.write("\17\3\17\3\17\3\17\5\17\u012d\n\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\5\17\u0136\n\17\3\17\3\17\3\17\5\17\u013b")
        buf.write("\n\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\7\17\u0144\n")
        buf.write("\17\f\17\16\17\u0147\13\17\3\17\3\17\3\17\3\17\3\17\3")
        buf.write("\17\3\17\3\17\3\17\5\17\u0152\n\17\3\17\3\17\3\17\3\17")
        buf.write("\5\17\u0158\n\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3")
        buf.write("\17\3\17\3\17\3\17\5\17\u0165\n\17\3\17\3\17\3\17\7\17")
        buf.write("\u016a\n\17\f\17\16\17\u016d\13\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17\3\17")
        buf.write("\3\17\3\17\3\17\3\17\3\17\3\17\3\17\7\17\u0185\n\17\f")
        buf.write("\17\16\17\u0188\13\17\3\17\3\17\3\17\5\17\u018d\n\17\3")
        buf.write("\17\5\17\u0190\n\17\3\20\3\20\3\20\3\20\3\20\3\20\3\20")
        buf.write("\3\20\3\20\3\20\7\20\u019c\n\20\f\20\16\20\u019f\13\20")
        buf.write("\3\21\3\21\5\21\u01a3\n\21\3\21\3\21\3\21\5\21\u01a8\n")
        buf.write("\21\3\21\3\21\3\21\3\21\3\21\5\21\u01af\n\21\3\22\3\22")
        buf.write("\3\22\5\22\u01b4\n\22\3\22\3\22\3\22\5\22\u01b9\n\22\3")
        buf.write("\22\3\22\3\22\5\22\u01be\n\22\3\22\3\22\3\22\3\22\5\22")
        buf.write("\u01c4\n\22\3\22\3\22\5\22\u01c8\n\22\3\23\3\23\3\23\3")
        buf.write("\23\3\23\3\23\3\23\3\23\3\23\3\23\3\24\3\24\7\24\u01d6")
        buf.write("\n\24\f\24\16\24\u01d9\13\24\3\24\3\24\3\24\5\24\u01de")
        buf.write("\n\24\3\25\3\25\5\25\u01e2\n\25\3\25\3\25\3\25\5\25\u01e7")
        buf.write("\n\25\3\25\5\25\u01ea\n\25\3\26\3\26\5\26\u01ee\n\26\3")
        buf.write("\26\3\26\3\26\5\26\u01f3\n\26\3\26\3\26\3\26\5\26\u01f8")
        buf.write("\n\26\3\26\3\26\5\26\u01fc\n\26\3\26\3\26\3\26\3\26\5")
        buf.write("\26\u0202\n\26\3\26\3\26\5\26\u0206\n\26\3\26\3\26\5\26")
        buf.write("\u020a\n\26\3\26\5\26\u020d\n\26\5\26\u020f\n\26\3\26")
        buf.write("\3\26\5\26\u0213\n\26\3\27\3\27\3\27\5\27\u0218\n\27\3")
        buf.write("\27\3\27\3\27\3\27\3\27\3\27\3\27\3\27\5\27\u0222\n\27")
        buf.write("\3\27\3\27\3\27\5\27\u0227\n\27\3\27\3\27\3\27\5\27\u022c")
        buf.write("\n\27\3\27\3\27\3\27\3\27\3\27\5\27\u0233\n\27\3\30\3")
        buf.write("\30\3\30\3\30\3\30\3\30\3\30\3\30\3\30\3\30\5\30\u023f")
        buf.write("\n\30\3\31\3\31\3\31\5\31\u0244\n\31\3\31\3\31\3\31\3")
        buf.write("\31\3\31\3\31\3\31\3\31\3\31\5\31\u024f\n\31\3\31\3\31")
        buf.write("\3\31\3\31\5\31\u0255\n\31\3\31\3\31\5\31\u0259\n\31\3")
        buf.write("\31\3\31\3\31\5\31\u025e\n\31\3\31\3\31\3\31\5\31\u0263")
        buf.write("\n\31\3\31\3\31\3\31\3\31\3\31\3\31\3\31\5\31\u026c\n")
        buf.write("\31\3\31\5\31\u026f\n\31\3\31\5\31\u0272\n\31\3\32\3\32")
        buf.write("\3\32\3\32\3\32\3\32\3\32\3\32\3\32\3\32\5\32\u027e\n")
        buf.write("\32\3\32\3\32\3\32\3\32\3\32\5\32\u0285\n\32\3\33\3\33")
        buf.write("\3\33\5\33\u028a\n\33\3\33\3\33\3\33\3\33\3\33\3\33\3")
        buf.write("\33\3\33\3\33\3\33\5\33\u0296\n\33\3\33\3\33\3\33\5\33")
        buf.write("\u029b\n\33\3\33\3\33\3\33\5\33\u02a0\n\33\3\33\3\33\3")
        buf.write("\33\5\33\u02a5\n\33\3\33\3\33\3\33\5\33\u02aa\n\33\3\33")
        buf.write("\3\33\3\33\3\33\3\33\3\33\3\33\5\33\u02b3\n\33\3\34\3")
        buf.write("\34\3\34\3\34\3\34\3\34\3\34\3\34\3\34\3\35\3\35\3\35")
        buf.write("\3\35\3\35\3\36\3\36\3\36\3\36\3\36\3\36\3\36\3\36\3\36")
        buf.write("\3\36\3\36\5\36\u02ce\n\36\3\36\3\36\3\36\5\36\u02d3\n")
        buf.write("\36\3\36\3\36\3\36\3\36\5\36\u02d9\n\36\3\36\3\36\3\36")
        buf.write("\3\36\3\36\3\36\3\36\3\36\5\36\u02e3\n\36\3\36\5\36\u02e6")
        buf.write("\n\36\3\36\3\36\3\36\5\36\u02eb\n\36\3\37\3\37\3\37\5")
        buf.write("\37\u02f0\n\37\3\37\3\37\5\37\u02f4\n\37\3 \3 \3 \3 \3")
        buf.write("!\3!\3!\3!\3!\3!\5!\u0300\n!\3!\3!\3\"\3\"\3\"\3#\3#\3")
        buf.write("#\3$\3$\3$\3%\3%\3%\3&\3&\3&\3&\3&\3&\3&\3&\5&\u0318\n")
        buf.write("&\3\'\3\'\3\'\3(\3(\3(\5(\u0320\n(\3(\3(\3(\3(\3(\5(\u0327")
        buf.write("\n(\3(\5(\u032a\n(\3(\5(\u032d\n(\3(\3(\3(\5(\u0332\n")
        buf.write("(\3(\3(\3(\5(\u0337\n(\3(\3(\3(\5(\u033c\n(\3(\3(\3(\3")
        buf.write("(\3(\3(\3(\3(\5(\u0346\n(\3(\3(\3(\5(\u034b\n(\3(\3(\3")
        buf.write("(\5(\u0350\n(\3(\3(\5(\u0354\n(\3(\3(\3(\3(\3(\3(\3(\5")
        buf.write("(\u035d\n(\3(\5(\u0360\n(\3(\3(\3(\3(\3(\5(\u0367\n(\3")
        buf.write("(\5(\u036a\n(\3(\5(\u036d\n(\3(\5(\u0370\n(\3)\3)\3)\3")
        buf.write(")\3)\3)\3)\3)\3)\3)\5)\u037c\n)\3)\3)\3)\3)\3)\5)\u0383")
        buf.write("\n)\3)\3)\3)\3)\3)\5)\u038a\n)\3*\3*\3*\5*\u038f\n*\3")
        buf.write("*\3*\3*\3*\3*\3*\3*\3*\3*\3*\5*\u039b\n*\3*\3*\3*\5*\u03a0")
        buf.write("\n*\3*\3*\3*\5*\u03a5\n*\3*\3*\3*\3*\3*\5*\u03ac\n*\3")
        buf.write("+\3+\3+\3+\3+\3+\3+\3+\3+\3+\3,\3,\3,\5,\u03bb\n,\3,\3")
        buf.write(",\3,\3,\3,\3,\3,\3,\3,\3,\3,\3,\5,\u03c9\n,\3,\3,\3,\5")
        buf.write(",\u03ce\n,\3,\3,\3,\5,\u03d3\n,\3,\3,\3,\3,\3,\5,\u03da")
        buf.write("\n,\3-\3-\3-\3-\3-\3-\3-\3-\3-\3-\3-\3-\3.\3.\3.\5.\u03eb")
        buf.write("\n.\3.\3.\3.\5.\u03f0\n.\3.\3.\3.\3.\3.\3.\3.\5.\u03f9")
        buf.write("\n.\3.\3.\3.\5.\u03fe\n.\3.\3.\3.\5.\u0403\n.\3.\3.\3")
        buf.write(".\5.\u0408\n.\3.\3.\3.\3.\3.\3.\3.\5.\u0411\n.\3.\5.\u0414")
        buf.write("\n.\3.\3.\3.\3.\3.\3.\5.\u041c\n.\3/\3/\3/\3/\3/\3/\3")
        buf.write("/\3/\3/\3/\3/\3/\3/\5/\u042b\n/\3\60\3\60\3\60\3\60\3")
        buf.write("\61\3\61\3\61\7\61\u0434\n\61\f\61\16\61\u0437\13\61\3")
        buf.write("\62\3\62\3\62\7\62\u043c\n\62\f\62\16\62\u043f\13\62\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\5\63\u044d\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\5\63\u045d\n\63")
        buf.write("\5\63\u045f\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\5\63")
        buf.write("\u0472\n\63\5\63\u0474\n\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\5\63\u047d\n\63\3\63\3\63\5\63\u0481\n\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\5\63\u048c\n")
        buf.write("\63\3\63\3\63\5\63\u0490\n\63\3\63\3\63\5\63\u0494\n\63")
        buf.write("\3\63\5\63\u0497\n\63\3\63\3\63\3\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\5\63\u04a4\n\63\5\63\u04a6\n\63")
        buf.write("\3\63\3\63\3\63\3\63\3\63\5\63\u04ad\n\63\5\63\u04af\n")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\5\63\u04b6\n\63\3\63\3\63")
        buf.write("\3\63\5\63\u04bb\n\63\5\63\u04bd\n\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63\3\63")
        buf.write("\3\63\3\63\3\63\3\63\5\63\u04d1\n\63\3\63\3\63\3\63\3")
        buf.write("\63\3\63\3\63\7\63\u04d9\n\63\f\63\16\63\u04dc\13\63\3")
        buf.write("\64\3\64\3\65\3\65\3\66\3\66\3\67\3\67\3\67\7\67\u04e7")
        buf.write("\n\67\f\67\16\67\u04ea\13\67\3\67\5\67\u04ed\n\67\5\67")
        buf.write("\u04ef\n\67\3\67\3\67\7\67\u04f3\n\67\f\67\16\67\u04f6")
        buf.write("\13\67\3\67\3\67\3\67\5\u0145\u016b\u0186\3d8\2\4\6\b")
        buf.write("\n\f\16\20\22\24\26\30\32\34\36 \"$&(*,.\60\62\64\668")
        buf.write(":<>@BDFHJLNPRTVXZ\\^`bdfhjl\2\5\3\2\u0088\u0089\4\2\u0084")
        buf.write("\u0084\u008a\u008a\4\2\u0084\u0084\u0086\u0086\2\u05f7")
        buf.write("\2}\3\2\2\2\4\u00a0\3\2\2\2\6\u00a9\3\2\2\2\b\u00b3\3")
        buf.write("\2\2\2\n\u00bd\3\2\2\2\f\u00c7\3\2\2\2\16\u00d0\3\2\2")
        buf.write("\2\20\u00d9\3\2\2\2\22\u00e2\3\2\2\2\24\u00eb\3\2\2\2")
        buf.write("\26\u00f4\3\2\2\2\30\u00fd\3\2\2\2\32\u0106\3\2\2\2\34")
        buf.write("\u018f\3\2\2\2\36\u0191\3\2\2\2 \u01ae\3\2\2\2\"\u01c7")
        buf.write("\3\2\2\2$\u01c9\3\2\2\2&\u01dd\3\2\2\2(\u01e9\3\2\2\2")
        buf.write("*\u0212\3\2\2\2,\u0232\3\2\2\2.\u0234\3\2\2\2\60\u0271")
        buf.write("\3\2\2\2\62\u0273\3\2\2\2\64\u02b2\3\2\2\2\66\u02b4\3")
        buf.write("\2\2\28\u02bd\3\2\2\2:\u02ea\3\2\2\2<\u02ec\3\2\2\2>\u02f5")
        buf.write("\3\2\2\2@\u02f9\3\2\2\2B\u0303\3\2\2\2D\u0306\3\2\2\2")
        buf.write("F\u0309\3\2\2\2H\u030c\3\2\2\2J\u030f\3\2\2\2L\u0319\3")
        buf.write("\2\2\2N\u036f\3\2\2\2P\u0371\3\2\2\2R\u03ab\3\2\2\2T\u03ad")
        buf.write("\3\2\2\2V\u03d9\3\2\2\2X\u03db\3\2\2\2Z\u041b\3\2\2\2")
        buf.write("\\\u041d\3\2\2\2^\u042c\3\2\2\2`\u0430\3\2\2\2b\u0438")
        buf.write("\3\2\2\2d\u04d0\3\2\2\2f\u04dd\3\2\2\2h\u04df\3\2\2\2")
        buf.write("j\u04e1\3\2\2\2l\u04e3\3\2\2\2n|\5\4\3\2o|\5\6\4\2p|\5")
        buf.write("\b\5\2q|\5\n\6\2r|\5\f\7\2s|\5\16\b\2t|\5\20\t\2u|\5\22")
        buf.write("\n\2v|\5\24\13\2w|\5\26\f\2x|\5\30\r\2y|\5\32\16\2z|\5")
        buf.write("l\67\2{n\3\2\2\2{o\3\2\2\2{p\3\2\2\2{q\3\2\2\2{r\3\2\2")
        buf.write("\2{s\3\2\2\2{t\3\2\2\2{u\3\2\2\2{v\3\2\2\2{w\3\2\2\2{")
        buf.write("x\3\2\2\2{y\3\2\2\2{z\3\2\2\2|\177\3\2\2\2}{\3\2\2\2}")
        buf.write("~\3\2\2\2~\u0083\3\2\2\2\177}\3\2\2\2\u0080\u0082\5\36")
        buf.write("\20\2\u0081\u0080\3\2\2\2\u0082\u0085\3\2\2\2\u0083\u0081")
        buf.write("\3\2\2\2\u0083\u0084\3\2\2\2\u0084\u0089\3\2\2\2\u0085")
        buf.write("\u0083\3\2\2\2\u0086\u0088\5$\23\2\u0087\u0086\3\2\2\2")
        buf.write("\u0088\u008b\3\2\2\2\u0089\u0087\3\2\2\2\u0089\u008a\3")
        buf.write("\2\2\2\u008a\u008f\3\2\2\2\u008b\u0089\3\2\2\2\u008c\u008e")
        buf.write("\5.\30\2\u008d\u008c\3\2\2\2\u008e\u0091\3\2\2\2\u008f")
        buf.write("\u008d\3\2\2\2\u008f\u0090\3\2\2\2\u0090\u0095\3\2\2\2")
        buf.write("\u0091\u008f\3\2\2\2\u0092\u0094\5&\24\2\u0093\u0092\3")
        buf.write("\2\2\2\u0094\u0097\3\2\2\2\u0095\u0093\3\2\2\2\u0095\u0096")
        buf.write("\3\2\2\2\u0096\u009b\3\2\2\2\u0097\u0095\3\2\2\2\u0098")
        buf.write("\u009a\5\62\32\2\u0099\u0098\3\2\2\2\u009a\u009d\3\2\2")
        buf.write("\2\u009b\u0099\3\2\2\2\u009b\u009c\3\2\2\2\u009c\u009e")
        buf.write("\3\2\2\2\u009d\u009b\3\2\2\2\u009e\u009f\7\2\2\3\u009f")
        buf.write("\3\3\2\2\2\u00a0\u00a4\7\5\2\2\u00a1\u00a3\5\34\17\2\u00a2")
        buf.write("\u00a1\3\2\2\2\u00a3\u00a6\3\2\2\2\u00a4\u00a2\3\2\2\2")
        buf.write("\u00a4\u00a5\3\2\2\2\u00a5\u00a7\3\2\2\2\u00a6\u00a4\3")
        buf.write("\2\2\2\u00a7\u00a8\7\4\2\2\u00a8\5\3\2\2\2\u00a9\u00aa")
        buf.write("\7\60\2\2\u00aa\u00ae\7\61\2\2\u00ab\u00ad\5\"\22\2\u00ac")
        buf.write("\u00ab\3\2\2\2\u00ad\u00b0\3\2\2\2\u00ae\u00ac\3\2\2\2")
        buf.write("\u00ae\u00af\3\2\2\2\u00af\u00b1\3\2\2\2\u00b0\u00ae\3")
        buf.write("\2\2\2\u00b1\u00b2\7\4\2\2\u00b2\7\3\2\2\2\u00b3\u00b4")
        buf.write("\7\60\2\2\u00b4\u00b8\7\64\2\2\u00b5\u00b7\5&\24\2\u00b6")
        buf.write("\u00b5\3\2\2\2\u00b7\u00ba\3\2\2\2\u00b8\u00b6\3\2\2\2")
        buf.write("\u00b8\u00b9\3\2\2\2\u00b9\u00bb\3\2\2\2\u00ba\u00b8\3")
        buf.write("\2\2\2\u00bb\u00bc\7\4\2\2\u00bc\t\3\2\2\2\u00bd\u00be")
        buf.write("\7\60\2\2\u00be\u00c2\78\2\2\u00bf\u00c1\5*\26\2\u00c0")
        buf.write("\u00bf\3\2\2\2\u00c1\u00c4\3\2\2\2\u00c2\u00c0\3\2\2\2")
        buf.write("\u00c2\u00c3\3\2\2\2\u00c3\u00c5\3\2\2\2\u00c4\u00c2\3")
        buf.write("\2\2\2\u00c5\u00c6\7\4\2\2\u00c6\13\3\2\2\2\u00c7\u00cb")
        buf.write("\7;\2\2\u00c8\u00ca\5,\27\2\u00c9\u00c8\3\2\2\2\u00ca")
        buf.write("\u00cd\3\2\2\2\u00cb\u00c9\3\2\2\2\u00cb\u00cc\3\2\2\2")
        buf.write("\u00cc\u00ce\3\2\2\2\u00cd\u00cb\3\2\2\2\u00ce\u00cf\7")
        buf.write("\4\2\2\u00cf\r\3\2\2\2\u00d0\u00d4\7?\2\2\u00d1\u00d3")
        buf.write("\5\60\31\2\u00d2\u00d1\3\2\2\2\u00d3\u00d6\3\2\2\2\u00d4")
        buf.write("\u00d2\3\2\2\2\u00d4\u00d5\3\2\2\2\u00d5\u00d7\3\2\2\2")
        buf.write("\u00d6\u00d4\3\2\2\2\u00d7\u00d8\7\4\2\2\u00d8\17\3\2")
        buf.write("\2\2\u00d9\u00dd\7A\2\2\u00da\u00dc\5\64\33\2\u00db\u00da")
        buf.write("\3\2\2\2\u00dc\u00df\3\2\2\2\u00dd\u00db\3\2\2\2\u00dd")
        buf.write("\u00de\3\2\2\2\u00de\u00e0\3\2\2\2\u00df\u00dd\3\2\2\2")
        buf.write("\u00e0\u00e1\7\4\2\2\u00e1\21\3\2\2\2\u00e2\u00e6\7G\2")
        buf.write("\2\u00e3\u00e5\5:\36\2\u00e4\u00e3\3\2\2\2\u00e5\u00e8")
        buf.write("\3\2\2\2\u00e6\u00e4\3\2\2\2\u00e6\u00e7\3\2\2\2\u00e7")
        buf.write("\u00e9\3\2\2\2\u00e8\u00e6\3\2\2\2\u00e9\u00ea\7\4\2\2")
        buf.write("\u00ea\23\3\2\2\2\u00eb\u00ef\7Q\2\2\u00ec\u00ee\5N(\2")
        buf.write("\u00ed\u00ec\3\2\2\2\u00ee\u00f1\3\2\2\2\u00ef\u00ed\3")
        buf.write("\2\2\2\u00ef\u00f0\3\2\2\2\u00f0\u00f2\3\2\2\2\u00f1\u00ef")
        buf.write("\3\2\2\2\u00f2\u00f3\7\4\2\2\u00f3\25\3\2\2\2\u00f4\u00f8")
        buf.write("\7V\2\2\u00f5\u00f7\5R*\2\u00f6\u00f5\3\2\2\2\u00f7\u00fa")
        buf.write("\3\2\2\2\u00f8\u00f6\3\2\2\2\u00f8\u00f9\3\2\2\2\u00f9")
        buf.write("\u00fb\3\2\2\2\u00fa\u00f8\3\2\2\2\u00fb\u00fc\7\4\2\2")
        buf.write("\u00fc\27\3\2\2\2\u00fd\u0101\7W\2\2\u00fe\u0100\5V,\2")
        buf.write("\u00ff\u00fe\3\2\2\2\u0100\u0103\3\2\2\2\u0101\u00ff\3")
        buf.write("\2\2\2\u0101\u0102\3\2\2\2\u0102\u0104\3\2\2\2\u0103\u0101")
        buf.write("\3\2\2\2\u0104\u0105\7\4\2\2\u0105\31\3\2\2\2\u0106\u010a")
        buf.write("\7X\2\2\u0107\u0109\5Z.\2\u0108\u0107\3\2\2\2\u0109\u010c")
        buf.write("\3\2\2\2\u010a\u0108\3\2\2\2\u010a\u010b\3\2\2\2\u010b")
        buf.write("\u010d\3\2\2\2\u010c\u010a\3\2\2\2\u010d\u010e\7\4\2\2")
        buf.write("\u010e\33\3\2\2\2\u010f\u0111\7\6\2\2\u0110\u0112\7\u008e")
        buf.write("\2\2\u0111\u0110\3\2\2\2\u0111\u0112\3\2\2\2\u0112\u0113")
        buf.write("\3\2\2\2\u0113\u0190\7%\2\2\u0114\u0190\5\36\20\2\u0115")
        buf.write("\u0116\7\b\2\2\u0116\u0117\7\u0088\2\2\u0117\u0190\5j")
        buf.write("\66\2\u0118\u0119\7\t\2\2\u0119\u011a\7\u0088\2\2\u011a")
        buf.write("\u0190\7*\2\2\u011b\u011c\7\n\2\2\u011c\u011d\7\u0088")
        buf.write("\2\2\u011d\u0190\5j\66\2\u011e\u0120\7\13\2\2\u011f\u0121")
        buf.write("\7\u008e\2\2\u0120\u011f\3\2\2\2\u0120\u0121\3\2\2\2\u0121")
        buf.write("\u0122\3\2\2\2\u0122\u0190\5j\66\2\u0123\u0125\7\f\2\2")
        buf.write("\u0124\u0126\7\u008e\2\2\u0125\u0124\3\2\2\2\u0125\u0126")
        buf.write("\3\2\2\2\u0126\u0127\3\2\2\2\u0127\u0190\7\u0088\2\2\u0128")
        buf.write("\u0129\7\r\2\2\u0129\u0190\7\u0088\2\2\u012a\u012c\7\16")
        buf.write("\2\2\u012b\u012d\7\u008e\2\2\u012c\u012b\3\2\2\2\u012c")
        buf.write("\u012d\3\2\2\2\u012d\u012e\3\2\2\2\u012e\u0190\7\u0084")
        buf.write("\2\2\u012f\u0130\7\17\2\2\u0130\u0190\7&\2\2\u0131\u0132")
        buf.write("\7\17\2\2\u0132\u0133\7\'\2\2\u0133\u0135\7(\2\2\u0134")
        buf.write("\u0136\7\u008e\2\2\u0135\u0134\3\2\2\2\u0135\u0136\3\2")
        buf.write("\2\2\u0136\u0137\3\2\2\2\u0137\u0138\5j\66\2\u0138\u013a")
        buf.write("\7)\2\2\u0139\u013b\7\u008e\2\2\u013a\u0139\3\2\2\2\u013a")
        buf.write("\u013b\3\2\2\2\u013b\u013c\3\2\2\2\u013c\u013d\5j\66\2")
        buf.write("\u013d\u0190\3\2\2\2\u013e\u013f\7\20\2\2\u013f\u0140")
        buf.write("\7\u0088\2\2\u0140\u0141\7\u0088\2\2\u0141\u0190\5j\66")
        buf.write("\2\u0142\u0144\7\21\2\2\u0143\u0142\3\2\2\2\u0144\u0147")
        buf.write("\3\2\2\2\u0145\u0146\3\2\2\2\u0145\u0143\3\2\2\2\u0146")
        buf.write("\u0148\3\2\2\2\u0147\u0145\3\2\2\2\u0148\u0190\7\4\2\2")
        buf.write("\u0149\u014a\7\22\2\2\u014a\u014b\7\u0088\2\2\u014b\u0190")
        buf.write("\7\u0084\2\2\u014c\u014d\7\23\2\2\u014d\u014e\7\u0088")
        buf.write("\2\2\u014e\u0190\7\u0084\2\2\u014f\u0151\7\24\2\2\u0150")
        buf.write("\u0152\7\u008e\2\2\u0151\u0150\3\2\2\2\u0151\u0152\3\2")
        buf.write("\2\2\u0152\u0153\3\2\2\2\u0153\u0190\7\u0084\2\2\u0154")
        buf.write("\u0190\7\25\2\2\u0155\u0157\7\26\2\2\u0156\u0158\7\u008e")
        buf.write("\2\2\u0157\u0156\3\2\2\2\u0157\u0158\3\2\2\2\u0158\u0159")
        buf.write("\3\2\2\2\u0159\u0190\7\u0084\2\2\u015a\u015b\7\27\2\2")
        buf.write("\u015b\u015c\7\u0088\2\2\u015c\u0190\7+\2\2\u015d\u015e")
        buf.write("\7\30\2\2\u015e\u015f\5 \21\2\u015f\u0160\7\4\2\2\u0160")
        buf.write("\u0190\3\2\2\2\u0161\u0162\7\31\2\2\u0162\u0164\7\33\2")
        buf.write("\2\u0163\u0165\7\u008e\2\2\u0164\u0163\3\2\2\2\u0164\u0165")
        buf.write("\3\2\2\2\u0165\u0166\3\2\2\2\u0166\u0190\5j\66\2\u0167")
        buf.write("\u016b\7\32\2\2\u0168\u016a\7\u0088\2\2\u0169\u0168\3")
        buf.write("\2\2\2\u016a\u016d\3\2\2\2\u016b\u016c\3\2\2\2\u016b\u0169")
        buf.write("\3\2\2\2\u016c\u016e\3\2\2\2\u016d\u016b\3\2\2\2\u016e")
        buf.write("\u0190\7\4\2\2\u016f\u0190\7\34\2\2\u0170\u0171\7\35\2")
        buf.write("\2\u0171\u0172\7\u0088\2\2\u0172\u0190\5j\66\2\u0173\u0174")
        buf.write("\7\36\2\2\u0174\u0175\7\u0088\2\2\u0175\u0190\5j\66\2")
        buf.write("\u0176\u0177\7\37\2\2\u0177\u0178\7\u0088\2\2\u0178\u0190")
        buf.write("\5j\66\2\u0179\u017a\7 \2\2\u017a\u017b\7\u0088\2\2\u017b")
        buf.write("\u0190\5j\66\2\u017c\u017d\7!\2\2\u017d\u017e\7\u0088")
        buf.write("\2\2\u017e\u0190\5j\66\2\u017f\u0180\7\"\2\2\u0180\u0181")
        buf.write("\7\u0088\2\2\u0181\u0190\5j\66\2\u0182\u0186\7#\2\2\u0183")
        buf.write("\u0185\7\u0088\2\2\u0184\u0183\3\2\2\2\u0185\u0188\3\2")
        buf.write("\2\2\u0186\u0187\3\2\2\2\u0186\u0184\3\2\2\2\u0187\u0189")
        buf.write("\3\2\2\2\u0188\u0186\3\2\2\2\u0189\u0190\7\4\2\2\u018a")
        buf.write("\u018c\7$\2\2\u018b\u018d\7\u008e\2\2\u018c\u018b\3\2")
        buf.write("\2\2\u018c\u018d\3\2\2\2\u018d\u018e\3\2\2\2\u018e\u0190")
        buf.write("\5j\66\2\u018f\u010f\3\2\2\2\u018f\u0114\3\2\2\2\u018f")
        buf.write("\u0115\3\2\2\2\u018f\u0118\3\2\2\2\u018f\u011b\3\2\2\2")
        buf.write("\u018f\u011e\3\2\2\2\u018f\u0123\3\2\2\2\u018f\u0128\3")
        buf.write("\2\2\2\u018f\u012a\3\2\2\2\u018f\u012f\3\2\2\2\u018f\u0131")
        buf.write("\3\2\2\2\u018f\u013e\3\2\2\2\u018f\u0145\3\2\2\2\u018f")
        buf.write("\u0149\3\2\2\2\u018f\u014c\3\2\2\2\u018f\u014f\3\2\2\2")
        buf.write("\u018f\u0154\3\2\2\2\u018f\u0155\3\2\2\2\u018f\u015a\3")
        buf.write("\2\2\2\u018f\u015d\3\2\2\2\u018f\u0161\3\2\2\2\u018f\u0167")
        buf.write("\3\2\2\2\u018f\u016f\3\2\2\2\u018f\u0170\3\2\2\2\u018f")
        buf.write("\u0173\3\2\2\2\u018f\u0176\3\2\2\2\u018f\u0179\3\2\2\2")
        buf.write("\u018f\u017c\3\2\2\2\u018f\u017f\3\2\2\2\u018f\u0182\3")
        buf.write("\2\2\2\u018f\u018a\3\2\2\2\u0190\35\3\2\2\2\u0191\u0192")
        buf.write("\7\7\2\2\u0192\u0193\5^\60\2\u0193\u0194\5^\60\2\u0194")
        buf.write("\u0195\5f\64\2\u0195\u0196\5f\64\2\u0196\u019d\5f\64\2")
        buf.write("\u0197\u0198\7\177\2\2\u0198\u0199\5^\60\2\u0199\u019a")
        buf.write("\5^\60\2\u019a\u019c\3\2\2\2\u019b\u0197\3\2\2\2\u019c")
        buf.write("\u019f\3\2\2\2\u019d\u019b\3\2\2\2\u019d\u019e\3\2\2\2")
        buf.write("\u019e\37\3\2\2\2\u019f\u019d\3\2\2\2\u01a0\u01a2\7,\2")
        buf.write("\2\u01a1\u01a3\7\u008e\2\2\u01a2\u01a1\3\2\2\2\u01a2\u01a3")
        buf.write("\3\2\2\2\u01a3\u01a4\3\2\2\2\u01a4\u01af\5j\66\2\u01a5")
        buf.write("\u01a7\7-\2\2\u01a6\u01a8\7\u008e\2\2\u01a7\u01a6\3\2")
        buf.write("\2\2\u01a7\u01a8\3\2\2\2\u01a8\u01a9\3\2\2\2\u01a9\u01af")
        buf.write("\5j\66\2\u01aa\u01ab\7.\2\2\u01ab\u01af\5^\60\2\u01ac")
        buf.write("\u01ad\7/\2\2\u01ad\u01af\5^\60\2\u01ae\u01a0\3\2\2\2")
        buf.write("\u01ae\u01a5\3\2\2\2\u01ae\u01aa\3\2\2\2\u01ae\u01ac\3")
        buf.write("\2\2\2\u01af!\3\2\2\2\u01b0\u01c8\5$\23\2\u01b1\u01b3")
        buf.write("\7\16\2\2\u01b2\u01b4\7\u008e\2\2\u01b3\u01b2\3\2\2\2")
        buf.write("\u01b3\u01b4\3\2\2\2\u01b4\u01b5\3\2\2\2\u01b5\u01c8\7")
        buf.write("\u0084\2\2\u01b6\u01b8\7\62\2\2\u01b7\u01b9\7\u008e\2")
        buf.write("\2\u01b8\u01b7\3\2\2\2\u01b8\u01b9\3\2\2\2\u01b9\u01ba")
        buf.write("\3\2\2\2\u01ba\u01c8\7\u0084\2\2\u01bb\u01bd\7\26\2\2")
        buf.write("\u01bc\u01be\7\u008e\2\2\u01bd\u01bc\3\2\2\2\u01bd\u01be")
        buf.write("\3\2\2\2\u01be\u01bf\3\2\2\2\u01bf\u01c8\7\u0084\2\2\u01c0")
        buf.write("\u01c8\7\34\2\2\u01c1\u01c3\7\36\2\2\u01c2\u01c4\7\u008e")
        buf.write("\2\2\u01c3\u01c2\3\2\2\2\u01c3\u01c4\3\2\2\2\u01c4\u01c5")
        buf.write("\3\2\2\2\u01c5\u01c8\5j\66\2\u01c6\u01c8\7\63\2\2\u01c7")
        buf.write("\u01b0\3\2\2\2\u01c7\u01b1\3\2\2\2\u01c7\u01b6\3\2\2\2")
        buf.write("\u01c7\u01bb\3\2\2\2\u01c7\u01c0\3\2\2\2\u01c7\u01c1\3")
        buf.write("\2\2\2\u01c7\u01c6\3\2\2\2\u01c8#\3\2\2\2\u01c9\u01ca")
        buf.write("\7\7\2\2\u01ca\u01cb\5^\60\2\u01cb\u01cc\5^\60\2\u01cc")
        buf.write("\u01cd\5^\60\2\u01cd\u01ce\5^\60\2\u01ce\u01cf\5f\64\2")
        buf.write("\u01cf\u01d0\5f\64\2\u01d0\u01d1\5f\64\2\u01d1\u01d2\7")
        buf.write("\u0084\2\2\u01d2%\3\2\2\2\u01d3\u01d7\7\65\2\2\u01d4\u01d6")
        buf.write("\5(\25\2\u01d5\u01d4\3\2\2\2\u01d6\u01d9\3\2\2\2\u01d7")
        buf.write("\u01d5\3\2\2\2\u01d7\u01d8\3\2\2\2\u01d8\u01da\3\2\2\2")
        buf.write("\u01d9\u01d7\3\2\2\2\u01da\u01de\7\4\2\2\u01db\u01de\7")
        buf.write("&\2\2\u01dc\u01de\7\63\2\2\u01dd\u01d3\3\2\2\2\u01dd\u01db")
        buf.write("\3\2\2\2\u01dd\u01dc\3\2\2\2\u01de\'\3\2\2\2\u01df\u01e1")
        buf.write("\7\66\2\2\u01e0\u01e2\7\u008e\2\2\u01e1\u01e0\3\2\2\2")
        buf.write("\u01e1\u01e2\3\2\2\2\u01e2\u01e3\3\2\2\2\u01e3\u01ea\5")
        buf.write("^\60\2\u01e4\u01e6\7\67\2\2\u01e5\u01e7\7\u008e\2\2\u01e6")
        buf.write("\u01e5\3\2\2\2\u01e6\u01e7\3\2\2\2\u01e7\u01e8\3\2\2\2")
        buf.write("\u01e8\u01ea\5j\66\2\u01e9\u01df\3\2\2\2\u01e9\u01e4\3")
        buf.write("\2\2\2\u01ea)\3\2\2\2\u01eb\u01ed\79\2\2\u01ec\u01ee\7")
        buf.write("\u008e\2\2\u01ed\u01ec\3\2\2\2\u01ed\u01ee\3\2\2\2\u01ee")
        buf.write("\u01ef\3\2\2\2\u01ef\u0213\7\u0084\2\2\u01f0\u01f2\7:")
        buf.write("\2\2\u01f1\u01f3\7\u008e\2\2\u01f2\u01f1\3\2\2\2\u01f2")
        buf.write("\u01f3\3\2\2\2\u01f3\u01f4\3\2\2\2\u01f4\u020e\7\u008b")
        buf.write("\2\2\u01f5\u01f7\5j\66\2\u01f6\u01f8\7\u0082\2\2\u01f7")
        buf.write("\u01f6\3\2\2\2\u01f7\u01f8\3\2\2\2\u01f8\u01f9\3\2\2\2")
        buf.write("\u01f9\u01fb\5j\66\2\u01fa\u01fc\7\u0082\2\2\u01fb\u01fa")
        buf.write("\3\2\2\2\u01fb\u01fc\3\2\2\2\u01fc\u01fd\3\2\2\2\u01fd")
        buf.write("\u01fe\5j\66\2\u01fe\u020f\3\2\2\2\u01ff\u0201\7}\2\2")
        buf.write("\u0200\u0202\7\u008e\2\2\u0201\u0200\3\2\2\2\u0201\u0202")
        buf.write("\3\2\2\2\u0202\u0203\3\2\2\2\u0203\u0205\5^\60\2\u0204")
        buf.write("\u0206\7\u0082\2\2\u0205\u0204\3\2\2\2\u0205\u0206\3\2")
        buf.write("\2\2\u0206\u020c\3\2\2\2\u0207\u0209\7~\2\2\u0208\u020a")
        buf.write("\7\u008e\2\2\u0209\u0208\3\2\2\2\u0209\u020a\3\2\2\2\u020a")
        buf.write("\u020b\3\2\2\2\u020b\u020d\5^\60\2\u020c\u0207\3\2\2\2")
        buf.write("\u020c\u020d\3\2\2\2\u020d\u020f\3\2\2\2\u020e\u01f5\3")
        buf.write("\2\2\2\u020e\u01ff\3\2\2\2\u020f\u0210\3\2\2\2\u0210\u0211")
        buf.write("\7\u008c\2\2\u0211\u0213\3\2\2\2\u0212\u01eb\3\2\2\2\u0212")
        buf.write("\u01f0\3\2\2\2\u0213+\3\2\2\2\u0214\u0233\5.\30\2\u0215")
        buf.write("\u0217\7\f\2\2\u0216\u0218\7\u008e\2\2\u0217\u0216\3\2")
        buf.write("\2\2\u0217\u0218\3\2\2\2\u0218\u0219\3\2\2\2\u0219\u0233")
        buf.write("\7\u0088\2\2\u021a\u021b\7<\2\2\u021b\u021c\5j\66\2\u021c")
        buf.write("\u021d\5j\66\2\u021d\u021e\5j\66\2\u021e\u0233\3\2\2\2")
        buf.write("\u021f\u0221\7=\2\2\u0220\u0222\7\u008e\2\2\u0221\u0220")
        buf.write("\3\2\2\2\u0221\u0222\3\2\2\2\u0222\u0223\3\2\2\2\u0223")
        buf.write("\u0233\5j\66\2\u0224\u0226\7\24\2\2\u0225\u0227\7\u008e")
        buf.write("\2\2\u0226\u0225\3\2\2\2\u0226\u0227\3\2\2\2\u0227\u0228")
        buf.write("\3\2\2\2\u0228\u0233\7\u0084\2\2\u0229\u022b\7\27\2\2")
        buf.write("\u022a\u022c\7\u008e\2\2\u022b\u022a\3\2\2\2\u022b\u022c")
        buf.write("\3\2\2\2\u022c\u022d\3\2\2\2\u022d\u0233\7>\2\2\u022e")
        buf.write("\u022f\7\31\2\2\u022f\u0230\7\33\2\2\u0230\u0233\5j\66")
        buf.write("\2\u0231\u0233\7\34\2\2\u0232\u0214\3\2\2\2\u0232\u0215")
        buf.write("\3\2\2\2\u0232\u021a\3\2\2\2\u0232\u021f\3\2\2\2\u0232")
        buf.write("\u0224\3\2\2\2\u0232\u0229\3\2\2\2\u0232\u022e\3\2\2\2")
        buf.write("\u0232\u0231\3\2\2\2\u0233-\3\2\2\2\u0234\u0235\7\7\2")
        buf.write("\2\u0235\u0236\5^\60\2\u0236\u0237\5^\60\2\u0237\u0238")
        buf.write("\5^\60\2\u0238\u0239\5^\60\2\u0239\u023a\5^\60\2\u023a")
        buf.write("\u023b\5^\60\2\u023b\u023c\5f\64\2\u023c\u023e\5f\64\2")
        buf.write("\u023d\u023f\5f\64\2\u023e\u023d\3\2\2\2\u023e\u023f\3")
        buf.write("\2\2\2\u023f/\3\2\2\2\u0240\u0272\5\62\32\2\u0241\u0243")
        buf.write("\7\f\2\2\u0242\u0244\7\u008e\2\2\u0243\u0242\3\2\2\2\u0243")
        buf.write("\u0244\3\2\2\2\u0244\u0245\3\2\2\2\u0245\u0272\7\u0088")
        buf.write("\2\2\u0246\u0247\7<\2\2\u0247\u0248\5j\66\2\u0248\u0249")
        buf.write("\5j\66\2\u0249\u024a\5j\66\2\u024a\u024b\5j\66\2\u024b")
        buf.write("\u0272\3\2\2\2\u024c\u024e\7\16\2\2\u024d\u024f\7\u008e")
        buf.write("\2\2\u024e\u024d\3\2\2\2\u024e\u024f\3\2\2\2\u024f\u0250")
        buf.write("\3\2\2\2\u0250\u0272\7\u0084\2\2\u0251\u0252\7=\2\2\u0252")
        buf.write("\u0254\5j\66\2\u0253\u0255\5j\66\2\u0254\u0253\3\2\2\2")
        buf.write("\u0254\u0255\3\2\2\2\u0255\u0272\3\2\2\2\u0256\u0258\7")
        buf.write("\24\2\2\u0257\u0259\7\u008e\2\2\u0258\u0257\3\2\2\2\u0258")
        buf.write("\u0259\3\2\2\2\u0259\u025a\3\2\2\2\u025a\u0272\7\u0084")
        buf.write("\2\2\u025b\u025d\7\26\2\2\u025c\u025e\7\u008e\2\2\u025d")
        buf.write("\u025c\3\2\2\2\u025d\u025e\3\2\2\2\u025e\u025f\3\2\2\2")
        buf.write("\u025f\u0272\7\u0084\2\2\u0260\u0262\7\27\2\2\u0261\u0263")
        buf.write("\7\u008e\2\2\u0262\u0261\3\2\2\2\u0262\u0263\3\2\2\2\u0263")
        buf.write("\u0264\3\2\2\2\u0264\u0272\7@\2\2\u0265\u0266\7\31\2\2")
        buf.write("\u0266\u0267\7\33\2\2\u0267\u026e\5j\66\2\u0268\u026f")
        buf.write("\7\\\2\2\u0269\u026b\7\f\2\2\u026a\u026c\7\u008e\2\2\u026b")
        buf.write("\u026a\3\2\2\2\u026b\u026c\3\2\2\2\u026c\u026d\3\2\2\2")
        buf.write("\u026d\u026f\7\u0088\2\2\u026e\u0268\3\2\2\2\u026e\u0269")
        buf.write("\3\2\2\2\u026f\u0272\3\2\2\2\u0270\u0272\7\34\2\2\u0271")
        buf.write("\u0240\3\2\2\2\u0271\u0241\3\2\2\2\u0271\u0246\3\2\2\2")
        buf.write("\u0271\u024c\3\2\2\2\u0271\u0251\3\2\2\2\u0271\u0256\3")
        buf.write("\2\2\2\u0271\u025b\3\2\2\2\u0271\u0260\3\2\2\2\u0271\u0265")
        buf.write("\3\2\2\2\u0271\u0270\3\2\2\2\u0272\61\3\2\2\2\u0273\u0274")
        buf.write("\7\7\2\2\u0274\u0275\5^\60\2\u0275\u0276\5^\60\2\u0276")
        buf.write("\u0277\5^\60\2\u0277\u027d\5^\60\2\u0278\u0279\5^\60\2")
        buf.write("\u0279\u027a\5^\60\2\u027a\u027b\5^\60\2\u027b\u027c\5")
        buf.write("^\60\2\u027c\u027e\3\2\2\2\u027d\u0278\3\2\2\2\u027d\u027e")
        buf.write("\3\2\2\2\u027e\u027f\3\2\2\2\u027f\u0280\5f\64\2\u0280")
        buf.write("\u0284\5f\64\2\u0281\u0282\5f\64\2\u0282\u0283\5f\64\2")
        buf.write("\u0283\u0285\3\2\2\2\u0284\u0281\3\2\2\2\u0284\u0285\3")
        buf.write("\2\2\2\u0285\63\3\2\2\2\u0286\u02b3\5\66\34\2\u0287\u0289")
        buf.write("\7\f\2\2\u0288\u028a\7\u008e\2\2\u0289\u0288\3\2\2\2\u0289")
        buf.write("\u028a\3\2\2\2\u028a\u028b\3\2\2\2\u028b\u02b3\7\u0088")
        buf.write("\2\2\u028c\u028d\7B\2\2\u028d\u028e\7\u0084\2\2\u028e")
        buf.write("\u028f\7\u0084\2\2\u028f\u0290\5j\66\2\u0290\u0291\5j")
        buf.write("\66\2\u0291\u0292\5j\66\2\u0292\u02b3\3\2\2\2\u0293\u0295")
        buf.write("\7=\2\2\u0294\u0296\7\u008e\2\2\u0295\u0294\3\2\2\2\u0295")
        buf.write("\u0296\3\2\2\2\u0296\u0297\3\2\2\2\u0297\u02b3\5j\66\2")
        buf.write("\u0298\u029a\7\24\2\2\u0299\u029b\7\u008e\2\2\u029a\u0299")
        buf.write("\3\2\2\2\u029a\u029b\3\2\2\2\u029b\u029c\3\2\2\2\u029c")
        buf.write("\u02b3\7\u0084\2\2\u029d\u029f\7C\2\2\u029e\u02a0\7\u008e")
        buf.write("\2\2\u029f\u029e\3\2\2\2\u029f\u02a0\3\2\2\2\u02a0\u02a1")
        buf.write("\3\2\2\2\u02a1\u02b3\5j\66\2\u02a2\u02a4\7D\2\2\u02a3")
        buf.write("\u02a5\7\u008e\2\2\u02a4\u02a3\3\2\2\2\u02a4\u02a5\3\2")
        buf.write("\2\2\u02a5\u02a6\3\2\2\2\u02a6\u02b3\5j\66\2\u02a7\u02a9")
        buf.write("\7\27\2\2\u02a8\u02aa\7\u008e\2\2\u02a9\u02a8\3\2\2\2")
        buf.write("\u02a9\u02aa\3\2\2\2\u02aa\u02ab\3\2\2\2\u02ab\u02b3\7")
        buf.write(">\2\2\u02ac\u02ad\7\31\2\2\u02ad\u02ae\7\33\2\2\u02ae")
        buf.write("\u02b3\5j\66\2\u02af\u02b3\58\35\2\u02b0\u02b3\7\34\2")
        buf.write("\2\u02b1\u02b3\7F\2\2\u02b2\u0286\3\2\2\2\u02b2\u0287")
        buf.write("\3\2\2\2\u02b2\u028c\3\2\2\2\u02b2\u0293\3\2\2\2\u02b2")
        buf.write("\u0298\3\2\2\2\u02b2\u029d\3\2\2\2\u02b2\u02a2\3\2\2\2")
        buf.write("\u02b2\u02a7\3\2\2\2\u02b2\u02ac\3\2\2\2\u02b2\u02af\3")
        buf.write("\2\2\2\u02b2\u02b0\3\2\2\2\u02b2\u02b1\3\2\2\2\u02b3\65")
        buf.write("\3\2\2\2\u02b4\u02b5\7\7\2\2\u02b5\u02b6\5^\60\2\u02b6")
        buf.write("\u02b7\5^\60\2\u02b7\u02b8\5^\60\2\u02b8\u02b9\5^\60\2")
        buf.write("\u02b9\u02ba\5^\60\2\u02ba\u02bb\5f\64\2\u02bb\u02bc\5")
        buf.write("f\64\2\u02bc\67\3\2\2\2\u02bd\u02be\7E\2\2\u02be\u02bf")
        buf.write("\5^\60\2\u02bf\u02c0\5j\66\2\u02c0\u02c1\5j\66\2\u02c1")
        buf.write("9\3\2\2\2\u02c2\u02eb\5<\37\2\u02c3\u02eb\5> \2\u02c4")
        buf.write("\u02eb\5@!\2\u02c5\u02eb\5B\"\2\u02c6\u02eb\5D#\2\u02c7")
        buf.write("\u02eb\5F$\2\u02c8\u02eb\5H%\2\u02c9\u02eb\5J&\2\u02ca")
        buf.write("\u02eb\5L\'\2\u02cb\u02cd\7\f\2\2\u02cc\u02ce\7\u008e")
        buf.write("\2\2\u02cd\u02cc\3\2\2\2\u02cd\u02ce\3\2\2\2\u02ce\u02cf")
        buf.write("\3\2\2\2\u02cf\u02eb\7\u0088\2\2\u02d0\u02d2\7P\2\2\u02d1")
        buf.write("\u02d3\7\u008e\2\2\u02d2\u02d1\3\2\2\2\u02d2\u02d3\3\2")
        buf.write("\2\2\u02d3\u02d4\3\2\2\2\u02d4\u02eb\5j\66\2\u02d5\u02d6")
        buf.write("\7=\2\2\u02d6\u02d8\5j\66\2\u02d7\u02d9\5j\66\2\u02d8")
        buf.write("\u02d7\3\2\2\2\u02d8\u02d9\3\2\2\2\u02d9\u02eb\3\2\2\2")
        buf.write("\u02da\u02db\7\27\2\2\u02db\u02eb\7@\2\2\u02dc\u02dd\7")
        buf.write("\31\2\2\u02dd\u02de\7\33\2\2\u02de\u02e5\5j\66\2\u02df")
        buf.write("\u02e6\7\\\2\2\u02e0\u02e2\7\f\2\2\u02e1\u02e3\7\u008e")
        buf.write("\2\2\u02e2\u02e1\3\2\2\2\u02e2\u02e3\3\2\2\2\u02e3\u02e4")
        buf.write("\3\2\2\2\u02e4\u02e6\7\u0088\2\2\u02e5\u02df\3\2\2\2\u02e5")
        buf.write("\u02e0\3\2\2\2\u02e6\u02e7\3\2\2\2\u02e7\u02e8\7\u0088")
        buf.write("\2\2\u02e8\u02eb\3\2\2\2\u02e9\u02eb\7\34\2\2\u02ea\u02c2")
        buf.write("\3\2\2\2\u02ea\u02c3\3\2\2\2\u02ea\u02c4\3\2\2\2\u02ea")
        buf.write("\u02c5\3\2\2\2\u02ea\u02c6\3\2\2\2\u02ea\u02c7\3\2\2\2")
        buf.write("\u02ea\u02c8\3\2\2\2\u02ea\u02c9\3\2\2\2\u02ea\u02ca\3")
        buf.write("\2\2\2\u02ea\u02cb\3\2\2\2\u02ea\u02d0\3\2\2\2\u02ea\u02d5")
        buf.write("\3\2\2\2\u02ea\u02da\3\2\2\2\u02ea\u02dc\3\2\2\2\u02ea")
        buf.write("\u02e9\3\2\2\2\u02eb;\3\2\2\2\u02ec\u02ed\7H\2\2\u02ed")
        buf.write("\u02ef\5^\60\2\u02ee\u02f0\5^\60\2\u02ef\u02ee\3\2\2\2")
        buf.write("\u02ef\u02f0\3\2\2\2\u02f0\u02f1\3\2\2\2\u02f1\u02f3\5")
        buf.write("j\66\2\u02f2\u02f4\5j\66\2\u02f3\u02f2\3\2\2\2\u02f3\u02f4")
        buf.write("\3\2\2\2\u02f4=\3\2\2\2\u02f5\u02f6\7E\2\2\u02f6\u02f7")
        buf.write("\5^\60\2\u02f7\u02f8\5j\66\2\u02f8?\3\2\2\2\u02f9\u02fa")
        buf.write("\7I\2\2\u02fa\u02fb\5^\60\2\u02fb\u02fc\5^\60\2\u02fc")
        buf.write("\u02fd\5^\60\2\u02fd\u02ff\7\u0088\2\2\u02fe\u0300\7\u0085")
        buf.write("\2\2\u02ff\u02fe\3\2\2\2\u02ff\u0300\3\2\2\2\u0300\u0301")
        buf.write("\3\2\2\2\u0301\u0302\7\u0088\2\2\u0302A\3\2\2\2\u0303")
        buf.write("\u0304\7J\2\2\u0304\u0305\5^\60\2\u0305C\3\2\2\2\u0306")
        buf.write("\u0307\7K\2\2\u0307\u0308\5^\60\2\u0308E\3\2\2\2\u0309")
        buf.write("\u030a\7L\2\2\u030a\u030b\5^\60\2\u030bG\3\2\2\2\u030c")
        buf.write("\u030d\7M\2\2\u030d\u030e\5^\60\2\u030eI\3\2\2\2\u030f")
        buf.write("\u0310\7N\2\2\u0310\u0311\7\u0088\2\2\u0311\u0312\5^\60")
        buf.write("\2\u0312\u0313\5^\60\2\u0313\u0314\5^\60\2\u0314\u0315")
        buf.write("\5^\60\2\u0315\u0317\5^\60\2\u0316\u0318\5^\60\2\u0317")
        buf.write("\u0316\3\2\2\2\u0317\u0318\3\2\2\2\u0318K\3\2\2\2\u0319")
        buf.write("\u031a\7O\2\2\u031a\u031b\5^\60\2\u031bM\3\2\2\2\u031c")
        buf.write("\u0370\5P)\2\u031d\u031f\7\f\2\2\u031e\u0320\7\u008e\2")
        buf.write("\2\u031f\u031e\3\2\2\2\u031f\u0320\3\2\2\2\u0320\u0321")
        buf.write("\3\2\2\2\u0321\u0370\7\u0088\2\2\u0322\u0370\7R\2\2\u0323")
        buf.write("\u0324\7B\2\2\u0324\u0326\7\u0084\2\2\u0325\u0327\7\u0084")
        buf.write("\2\2\u0326\u0325\3\2\2\2\u0326\u0327\3\2\2\2\u0327\u0329")
        buf.write("\3\2\2\2\u0328\u032a\7\u0084\2\2\u0329\u0328\3\2\2\2\u0329")
        buf.write("\u032a\3\2\2\2\u032a\u032c\3\2\2\2\u032b\u032d\7\u0084")
        buf.write("\2\2\u032c\u032b\3\2\2\2\u032c\u032d\3\2\2\2\u032d\u032e")
        buf.write("\3\2\2\2\u032e\u0370\5j\66\2\u032f\u0331\7P\2\2\u0330")
        buf.write("\u0332\7\u008e\2\2\u0331\u0330\3\2\2\2\u0331\u0332\3\2")
        buf.write("\2\2\u0332\u0333\3\2\2\2\u0333\u0370\5j\66\2\u0334\u0336")
        buf.write("\7=\2\2\u0335\u0337\7\u008e\2\2\u0336\u0335\3\2\2\2\u0336")
        buf.write("\u0337\3\2\2\2\u0337\u0338\3\2\2\2\u0338\u0370\5j\66\2")
        buf.write("\u0339\u033b\7\24\2\2\u033a\u033c\7\u008e\2\2\u033b\u033a")
        buf.write("\3\2\2\2\u033b\u033c\3\2\2\2\u033c\u033d\3\2\2\2\u033d")
        buf.write("\u0370\7\u0084\2\2\u033e\u033f\7S\2\2\u033f\u0340\7\u0084")
        buf.write("\2\2\u0340\u0341\7\u0084\2\2\u0341\u0345\7\u0084\2\2\u0342")
        buf.write("\u0343\7\u0084\2\2\u0343\u0344\7\u0084\2\2\u0344\u0346")
        buf.write("\7\u0084\2\2\u0345\u0342\3\2\2\2\u0345\u0346\3\2\2\2\u0346")
        buf.write("\u034a\3\2\2\2\u0347\u0348\7\u0084\2\2\u0348\u0349\7\u0084")
        buf.write("\2\2\u0349\u034b\7\u0084\2\2\u034a\u0347\3\2\2\2\u034a")
        buf.write("\u034b\3\2\2\2\u034b\u034f\3\2\2\2\u034c\u034d\7\u0084")
        buf.write("\2\2\u034d\u034e\7\u0084\2\2\u034e\u0350\7\u0084\2\2\u034f")
        buf.write("\u034c\3\2\2\2\u034f\u0350\3\2\2\2\u0350\u0370\3\2\2\2")
        buf.write("\u0351\u0353\7\27\2\2\u0352\u0354\7\u008e\2\2\u0353\u0352")
        buf.write("\3\2\2\2\u0353\u0354\3\2\2\2\u0354\u0355\3\2\2\2\u0355")
        buf.write("\u0370\7>\2\2\u0356\u0357\7\31\2\2\u0357\u0358\7\33\2")
        buf.write("\2\u0358\u035f\5j\66\2\u0359\u0360\7\\\2\2\u035a\u035c")
        buf.write("\7\f\2\2\u035b\u035d\7\u008e\2\2\u035c\u035b\3\2\2\2\u035c")
        buf.write("\u035d\3\2\2\2\u035d\u035e\3\2\2\2\u035e\u0360\7\u0088")
        buf.write("\2\2\u035f\u0359\3\2\2\2\u035f\u035a\3\2\2\2\u0360\u0370")
        buf.write("\3\2\2\2\u0361\u0370\7\34\2\2\u0362\u0363\7T\2\2\u0363")
        buf.write("\u0364\7U\2\2\u0364\u0366\7\u0084\2\2\u0365\u0367\7\u0084")
        buf.write("\2\2\u0366\u0365\3\2\2\2\u0366\u0367\3\2\2\2\u0367\u0369")
        buf.write("\3\2\2\2\u0368\u036a\7\u0084\2\2\u0369\u0368\3\2\2\2\u0369")
        buf.write("\u036a\3\2\2\2\u036a\u036c\3\2\2\2\u036b\u036d\7\u0084")
        buf.write("\2\2\u036c\u036b\3\2\2\2\u036c\u036d\3\2\2\2\u036d\u0370")
        buf.write("\3\2\2\2\u036e\u0370\7F\2\2\u036f\u031c\3\2\2\2\u036f")
        buf.write("\u031d\3\2\2\2\u036f\u0322\3\2\2\2\u036f\u0323\3\2\2\2")
        buf.write("\u036f\u032f\3\2\2\2\u036f\u0334\3\2\2\2\u036f\u0339\3")
        buf.write("\2\2\2\u036f\u033e\3\2\2\2\u036f\u0351\3\2\2\2\u036f\u0356")
        buf.write("\3\2\2\2\u036f\u0361\3\2\2\2\u036f\u0362\3\2\2\2\u036f")
        buf.write("\u036e\3\2\2\2\u0370O\3\2\2\2\u0371\u0372\7\7\2\2\u0372")
        buf.write("\u0373\5^\60\2\u0373\u0374\5^\60\2\u0374\u0375\5^\60\2")
        buf.write("\u0375\u037b\5^\60\2\u0376\u0377\5^\60\2\u0377\u0378\5")
        buf.write("^\60\2\u0378\u0379\5^\60\2\u0379\u037a\5^\60\2\u037a\u037c")
        buf.write("\3\2\2\2\u037b\u0376\3\2\2\2\u037b\u037c\3\2\2\2\u037c")
        buf.write("\u0382\3\2\2\2\u037d\u037e\5^\60\2\u037e\u037f\5^\60\2")
        buf.write("\u037f\u0380\5^\60\2\u0380\u0381\5^\60\2\u0381\u0383\3")
        buf.write("\2\2\2\u0382\u037d\3\2\2\2\u0382\u0383\3\2\2\2\u0383\u0389")
        buf.write("\3\2\2\2\u0384\u0385\5^\60\2\u0385\u0386\5^\60\2\u0386")
        buf.write("\u0387\5^\60\2\u0387\u0388\5^\60\2\u0388\u038a\3\2\2\2")
        buf.write("\u0389\u0384\3\2\2\2\u0389\u038a\3\2\2\2\u038aQ\3\2\2")
        buf.write("\2\u038b\u03ac\5T+\2\u038c\u038e\7\f\2\2\u038d\u038f\7")
        buf.write("\u008e\2\2\u038e\u038d\3\2\2\2\u038e\u038f\3\2\2\2\u038f")
        buf.write("\u0390\3\2\2\2\u0390\u03ac\7\u0088\2\2\u0391\u0392\7<")
        buf.write("\2\2\u0392\u0393\5j\66\2\u0393\u0394\5j\66\2\u0394\u0395")
        buf.write("\5j\66\2\u0395\u0396\5j\66\2\u0396\u0397\5j\66\2\u0397")
        buf.write("\u03ac\3\2\2\2\u0398\u039a\7=\2\2\u0399\u039b\7\u008e")
        buf.write("\2\2\u039a\u0399\3\2\2\2\u039a\u039b\3\2\2\2\u039b\u039c")
        buf.write("\3\2\2\2\u039c\u03ac\5j\66\2\u039d\u039f\7\24\2\2\u039e")
        buf.write("\u03a0\7\u008e\2\2\u039f\u039e\3\2\2\2\u039f\u03a0\3\2")
        buf.write("\2\2\u03a0\u03a1\3\2\2\2\u03a1\u03ac\7\u0084\2\2\u03a2")
        buf.write("\u03a4\7\27\2\2\u03a3\u03a5\7\u008e\2\2\u03a4\u03a3\3")
        buf.write("\2\2\2\u03a4\u03a5\3\2\2\2\u03a5\u03a6\3\2\2\2\u03a6\u03ac")
        buf.write("\7>\2\2\u03a7\u03a8\7\31\2\2\u03a8\u03a9\7\33\2\2\u03a9")
        buf.write("\u03ac\5j\66\2\u03aa\u03ac\7\34\2\2\u03ab\u038b\3\2\2")
        buf.write("\2\u03ab\u038c\3\2\2\2\u03ab\u0391\3\2\2\2\u03ab\u0398")
        buf.write("\3\2\2\2\u03ab\u039d\3\2\2\2\u03ab\u03a2\3\2\2\2\u03ab")
        buf.write("\u03a7\3\2\2\2\u03ab\u03aa\3\2\2\2\u03acS\3\2\2\2\u03ad")
        buf.write("\u03ae\7\7\2\2\u03ae\u03af\5^\60\2\u03af\u03b0\5^\60\2")
        buf.write("\u03b0\u03b1\5^\60\2\u03b1\u03b2\5^\60\2\u03b2\u03b3\5")
        buf.write("^\60\2\u03b3\u03b4\5^\60\2\u03b4\u03b5\5f\64\2\u03b5\u03b6")
        buf.write("\5f\64\2\u03b6U\3\2\2\2\u03b7\u03da\5X-\2\u03b8\u03ba")
        buf.write("\7\f\2\2\u03b9\u03bb\7\u008e\2\2\u03ba\u03b9\3\2\2\2\u03ba")
        buf.write("\u03bb\3\2\2\2\u03bb\u03bc\3\2\2\2\u03bc\u03da\7\u0088")
        buf.write("\2\2\u03bd\u03be\7<\2\2\u03be\u03bf\5j\66\2\u03bf\u03c0")
        buf.write("\5j\66\2\u03c0\u03c1\5j\66\2\u03c1\u03c2\5j\66\2\u03c2")
        buf.write("\u03c3\5j\66\2\u03c3\u03c4\5j\66\2\u03c4\u03c5\5j\66\2")
        buf.write("\u03c5\u03da\3\2\2\2\u03c6\u03c8\7=\2\2\u03c7\u03c9\7")
        buf.write("\u008e\2\2\u03c8\u03c7\3\2\2\2\u03c8\u03c9\3\2\2\2\u03c9")
        buf.write("\u03ca\3\2\2\2\u03ca\u03da\5j\66\2\u03cb\u03cd\7\24\2")
        buf.write("\2\u03cc\u03ce\7\u008e\2\2\u03cd\u03cc\3\2\2\2\u03cd\u03ce")
        buf.write("\3\2\2\2\u03ce\u03cf\3\2\2\2\u03cf\u03da\7\u0084\2\2\u03d0")
        buf.write("\u03d2\7\27\2\2\u03d1\u03d3\7\u008e\2\2\u03d2\u03d1\3")
        buf.write("\2\2\2\u03d2\u03d3\3\2\2\2\u03d3\u03d4\3\2\2\2\u03d4\u03da")
        buf.write("\7>\2\2\u03d5\u03d6\7\31\2\2\u03d6\u03d7\7\33\2\2\u03d7")
        buf.write("\u03da\5j\66\2\u03d8\u03da\7\34\2\2\u03d9\u03b7\3\2\2")
        buf.write("\2\u03d9\u03b8\3\2\2\2\u03d9\u03bd\3\2\2\2\u03d9\u03c6")
        buf.write("\3\2\2\2\u03d9\u03cb\3\2\2\2\u03d9\u03d0\3\2\2\2\u03d9")
        buf.write("\u03d5\3\2\2\2\u03d9\u03d8\3\2\2\2\u03daW\3\2\2\2\u03db")
        buf.write("\u03dc\7\7\2\2\u03dc\u03dd\5^\60\2\u03dd\u03de\5^\60\2")
        buf.write("\u03de\u03df\5^\60\2\u03df\u03e0\5^\60\2\u03e0\u03e1\5")
        buf.write("^\60\2\u03e1\u03e2\5^\60\2\u03e2\u03e3\5^\60\2\u03e3\u03e4")
        buf.write("\5^\60\2\u03e4\u03e5\5f\64\2\u03e5\u03e6\5f\64\2\u03e6")
        buf.write("Y\3\2\2\2\u03e7\u041c\5\\/\2\u03e8\u03ea\7\f\2\2\u03e9")
        buf.write("\u03eb\7\u008e\2\2\u03ea\u03e9\3\2\2\2\u03ea\u03eb\3\2")
        buf.write("\2\2\u03eb\u03ec\3\2\2\2\u03ec\u041c\7\u0088\2\2\u03ed")
        buf.write("\u03ef\7Y\2\2\u03ee\u03f0\7\u008e\2\2\u03ef\u03ee\3\2")
        buf.write("\2\2\u03ef\u03f0\3\2\2\2\u03f0\u03f1\3\2\2\2\u03f1\u041c")
        buf.write("\7\u0088\2\2\u03f2\u03f3\7B\2\2\u03f3\u03f4\7\u0084\2")
        buf.write("\2\u03f4\u03f5\7\u0084\2\2\u03f5\u041c\5j\66\2\u03f6\u03f8")
        buf.write("\7P\2\2\u03f7\u03f9\7\u008e\2\2\u03f8\u03f7\3\2\2\2\u03f8")
        buf.write("\u03f9\3\2\2\2\u03f9\u03fa\3\2\2\2\u03fa\u041c\5j\66\2")
        buf.write("\u03fb\u03fd\7=\2\2\u03fc\u03fe\7\u008e\2\2\u03fd\u03fc")
        buf.write("\3\2\2\2\u03fd\u03fe\3\2\2\2\u03fe\u03ff\3\2\2\2\u03ff")
        buf.write("\u041c\5j\66\2\u0400\u0402\7\24\2\2\u0401\u0403\7\u008e")
        buf.write("\2\2\u0402\u0401\3\2\2\2\u0402\u0403\3\2\2\2\u0403\u0404")
        buf.write("\3\2\2\2\u0404\u041c\7\u0084\2\2\u0405\u0407\7\27\2\2")
        buf.write("\u0406\u0408\7\u008e\2\2\u0407\u0406\3\2\2\2\u0407\u0408")
        buf.write("\3\2\2\2\u0408\u0409\3\2\2\2\u0409\u041c\7>\2\2\u040a")
        buf.write("\u040b\7\31\2\2\u040b\u040c\7\33\2\2\u040c\u0413\5j\66")
        buf.write("\2\u040d\u0414\7\\\2\2\u040e\u0410\7\f\2\2\u040f\u0411")
        buf.write("\7\u008e\2\2\u0410\u040f\3\2\2\2\u0410\u0411\3\2\2\2\u0411")
        buf.write("\u0412\3\2\2\2\u0412\u0414\7\u0088\2\2\u0413\u040d\3\2")
        buf.write("\2\2\u0413\u040e\3\2\2\2\u0414\u041c\3\2\2\2\u0415\u041c")
        buf.write("\7\34\2\2\u0416\u0417\7T\2\2\u0417\u0418\7Z\2\2\u0418")
        buf.write("\u0419\7\u0084\2\2\u0419\u041c\7\u0084\2\2\u041a\u041c")
        buf.write("\7F\2\2\u041b\u03e7\3\2\2\2\u041b\u03e8\3\2\2\2\u041b")
        buf.write("\u03ed\3\2\2\2\u041b\u03f2\3\2\2\2\u041b\u03f6\3\2\2\2")
        buf.write("\u041b\u03fb\3\2\2\2\u041b\u0400\3\2\2\2\u041b\u0405\3")
        buf.write("\2\2\2\u041b\u040a\3\2\2\2\u041b\u0415\3\2\2\2\u041b\u0416")
        buf.write("\3\2\2\2\u041b\u041a\3\2\2\2\u041c[\3\2\2\2\u041d\u041e")
        buf.write("\7\7\2\2\u041e\u041f\5^\60\2\u041f\u0420\5^\60\2\u0420")
        buf.write("\u0421\5^\60\2\u0421\u0422\5^\60\2\u0422\u0423\5^\60\2")
        buf.write("\u0423\u0424\5^\60\2\u0424\u0425\5^\60\2\u0425\u0426\5")
        buf.write("^\60\2\u0426\u0427\5^\60\2\u0427\u0428\5^\60\2\u0428\u042a")
        buf.write("\5^\60\2\u0429\u042b\5^\60\2\u042a\u0429\3\2\2\2\u042a")
        buf.write("\u042b\3\2\2\2\u042b]\3\2\2\2\u042c\u042d\7\u008b\2\2")
        buf.write("\u042d\u042e\5`\61\2\u042e\u042f\7\u008c\2\2\u042f_\3")
        buf.write("\2\2\2\u0430\u0435\5b\62\2\u0431\u0432\7\177\2\2\u0432")
        buf.write("\u0434\5b\62\2\u0433\u0431\3\2\2\2\u0434\u0437\3\2\2\2")
        buf.write("\u0435\u0433\3\2\2\2\u0435\u0436\3\2\2\2\u0436a\3\2\2")
        buf.write("\2\u0437\u0435\3\2\2\2\u0438\u043d\5d\63\2\u0439\u043a")
        buf.write("\7\u0080\2\2\u043a\u043c\5d\63\2\u043b\u0439\3\2\2\2\u043c")
        buf.write("\u043f\3\2\2\2\u043d\u043b\3\2\2\2\u043d\u043e\3\2\2\2")
        buf.write("\u043ec\3\2\2\2\u043f\u043d\3\2\2\2\u0440\u0441\b\63\1")
        buf.write("\2\u0441\u0442\7\u008b\2\2\u0442\u0443\5`\61\2\u0443\u0444")
        buf.write("\7\u008c\2\2\u0444\u04d1\3\2\2\2\u0445\u04d1\7\\\2\2\u0446")
        buf.write("\u0447\7^\2\2\u0447\u0448\t\2\2\2\u0448\u0449\t\3\2\2")
        buf.write("\u0449\u04d1\t\2\2\2\u044a\u044c\7_\2\2\u044b\u044d\7")
        buf.write("\u0098\2\2\u044c\u044b\3\2\2\2\u044c\u044d\3\2\2\2\u044d")
        buf.write("\u044e\3\2\2\2\u044e\u044f\7\u0099\2\2\u044f\u0450\7\u009a")
        buf.write("\2\2\u0450\u04d1\5h\65\2\u0451\u0452\7`\2\2\u0452\u04d1")
        buf.write("\5d\63 \u0453\u0454\7a\2\2\u0454\u04d1\5d\63\37\u0455")
        buf.write("\u0456\7b\2\2\u0456\u04d1\5d\63\36\u0457\u045e\7c\2\2")
        buf.write("\u0458\u045f\7\u0089\2\2\u0459\u045c\7\u0088\2\2\u045a")
        buf.write("\u045b\7\u008d\2\2\u045b\u045d\7\u0088\2\2\u045c\u045a")
        buf.write("\3\2\2\2\u045c\u045d\3\2\2\2\u045d\u045f\3\2\2\2\u045e")
        buf.write("\u0458\3\2\2\2\u045e\u0459\3\2\2\2\u045f\u04d1\3\2\2\2")
        buf.write("\u0460\u0461\7d\2\2\u0461\u0462\5h\65\2\u0462\u0463\5")
        buf.write("h\65\2\u0463\u0464\5h\65\2\u0464\u0465\5h\65\2\u0465\u0466")
        buf.write("\5h\65\2\u0466\u0467\5h\65\2\u0467\u04d1\3\2\2\2\u0468")
        buf.write("\u04d1\7e\2\2\u0469\u046a\7f\2\2\u046a\u04d1\7\u0084\2")
        buf.write("\2\u046b\u04d1\7g\2\2\u046c\u0473\7h\2\2\u046d\u0474\7")
        buf.write("\u0089\2\2\u046e\u0471\7\u0088\2\2\u046f\u0470\7\u008d")
        buf.write("\2\2\u0470\u0472\7\u0088\2\2\u0471\u046f\3\2\2\2\u0471")
        buf.write("\u0472\3\2\2\2\u0472\u0474\3\2\2\2\u0473\u046d\3\2\2\2")
        buf.write("\u0473\u046e\3\2\2\2\u0474\u04d1\3\2\2\2\u0475\u04d1\7")
        buf.write("i\2\2\u0476\u0477\7\u0081\2\2\u0477\u04d1\5d\63\26\u0478")
        buf.write("\u0479\7j\2\2\u0479\u047a\7\u008b\2\2\u047a\u047c\5h\65")
        buf.write("\2\u047b\u047d\7\u0082\2\2\u047c\u047b\3\2\2\2\u047c\u047d")
        buf.write("\3\2\2\2\u047d\u047e\3\2\2\2\u047e\u0480\5h\65\2\u047f")
        buf.write("\u0481\7\u0082\2\2\u0480\u047f\3\2\2\2\u0480\u0481\3\2")
        buf.write("\2\2\u0481\u0482\3\2\2\2\u0482\u0483\5h\65\2\u0483\u0484")
        buf.write("\7\u008c\2\2\u0484\u0485\7k\2\2\u0485\u0486\5h\65\2\u0486")
        buf.write("\u04d1\3\2\2\2\u0487\u0488\7j\2\2\u0488\u0489\7\u008b")
        buf.write("\2\2\u0489\u048b\7}\2\2\u048a\u048c\7\u008e\2\2\u048b")
        buf.write("\u048a\3\2\2\2\u048b\u048c\3\2\2\2\u048c\u048d\3\2\2\2")
        buf.write("\u048d\u048f\5^\60\2\u048e\u0490\7\u0082\2\2\u048f\u048e")
        buf.write("\3\2\2\2\u048f\u0490\3\2\2\2\u0490\u0496\3\2\2\2\u0491")
        buf.write("\u0493\7~\2\2\u0492\u0494\7\u008e\2\2\u0493\u0492\3\2")
        buf.write("\2\2\u0493\u0494\3\2\2\2\u0494\u0495\3\2\2\2\u0495\u0497")
        buf.write("\5^\60\2\u0496\u0491\3\2\2\2\u0496\u0497\3\2\2\2\u0497")
        buf.write("\u0498\3\2\2\2\u0498\u0499\7\u008c\2\2\u0499\u049a\7k")
        buf.write("\2\2\u049a\u049b\5h\65\2\u049b\u04d1\3\2\2\2\u049c\u04d1")
        buf.write("\7l\2\2\u049d\u04d1\7m\2\2\u049e\u04a5\7n\2\2\u049f\u04a6")
        buf.write("\7\u008a\2\2\u04a0\u04a3\7\u0084\2\2\u04a1\u04a2\7\u008d")
        buf.write("\2\2\u04a2\u04a4\7\u0084\2\2\u04a3\u04a1\3\2\2\2\u04a3")
        buf.write("\u04a4\3\2\2\2\u04a4\u04a6\3\2\2\2\u04a5\u049f\3\2\2\2")
        buf.write("\u04a5\u04a0\3\2\2\2\u04a6\u04d1\3\2\2\2\u04a7\u04ae\7")
        buf.write("o\2\2\u04a8\u04af\7\u0089\2\2\u04a9\u04ac\7\u0088\2\2")
        buf.write("\u04aa\u04ab\7\u008d\2\2\u04ab\u04ad\7\u0088\2\2\u04ac")
        buf.write("\u04aa\3\2\2\2\u04ac\u04ad\3\2\2\2\u04ad\u04af\3\2\2\2")
        buf.write("\u04ae\u04a8\3\2\2\2\u04ae\u04a9\3\2\2\2\u04af\u04d1\3")
        buf.write("\2\2\2\u04b0\u04bc\7q\2\2\u04b1\u04bd\7\u0089\2\2\u04b2")
        buf.write("\u04b5\7\u0088\2\2\u04b3\u04b4\7\u008d\2\2\u04b4\u04b6")
        buf.write("\7\u0088\2\2\u04b5\u04b3\3\2\2\2\u04b5\u04b6\3\2\2\2\u04b6")
        buf.write("\u04bd\3\2\2\2\u04b7\u04ba\7\u0087\2\2\u04b8\u04b9\7\u008d")
        buf.write("\2\2\u04b9\u04bb\7\u0087\2\2\u04ba\u04b8\3\2\2\2\u04ba")
        buf.write("\u04bb\3\2\2\2\u04bb\u04bd\3\2\2\2\u04bc\u04b1\3\2\2\2")
        buf.write("\u04bc\u04b2\3\2\2\2\u04bc\u04b7\3\2\2\2\u04bd\u04d1\3")
        buf.write("\2\2\2\u04be\u04bf\7r\2\2\u04bf\u04c0\5h\65\2\u04c0\u04c1")
        buf.write("\5h\65\2\u04c1\u04c2\5h\65\2\u04c2\u04c3\5h\65\2\u04c3")
        buf.write("\u04c4\5h\65\2\u04c4\u04c5\5h\65\2\u04c5\u04d1\3\2\2\2")
        buf.write("\u04c6\u04d1\7s\2\2\u04c7\u04d1\7t\2\2\u04c8\u04d1\7u")
        buf.write("\2\2\u04c9\u04d1\7v\2\2\u04ca\u04d1\7w\2\2\u04cb\u04d1")
        buf.write("\7x\2\2\u04cc\u04d1\7y\2\2\u04cd\u04d1\7z\2\2\u04ce\u04d1")
        buf.write("\7{\2\2\u04cf\u04d1\7|\2\2\u04d0\u0440\3\2\2\2\u04d0\u0445")
        buf.write("\3\2\2\2\u04d0\u0446\3\2\2\2\u04d0\u044a\3\2\2\2\u04d0")
        buf.write("\u0451\3\2\2\2\u04d0\u0453\3\2\2\2\u04d0\u0455\3\2\2\2")
        buf.write("\u04d0\u0457\3\2\2\2\u04d0\u0460\3\2\2\2\u04d0\u0468\3")
        buf.write("\2\2\2\u04d0\u0469\3\2\2\2\u04d0\u046b\3\2\2\2\u04d0\u046c")
        buf.write("\3\2\2\2\u04d0\u0475\3\2\2\2\u04d0\u0476\3\2\2\2\u04d0")
        buf.write("\u0478\3\2\2\2\u04d0\u0487\3\2\2\2\u04d0\u049c\3\2\2\2")
        buf.write("\u04d0\u049d\3\2\2\2\u04d0\u049e\3\2\2\2\u04d0\u04a7\3")
        buf.write("\2\2\2\u04d0\u04b0\3\2\2\2\u04d0\u04be\3\2\2\2\u04d0\u04c6")
        buf.write("\3\2\2\2\u04d0\u04c7\3\2\2\2\u04d0\u04c8\3\2\2\2\u04d0")
        buf.write("\u04c9\3\2\2\2\u04d0\u04ca\3\2\2\2\u04d0\u04cb\3\2\2\2")
        buf.write("\u04d0\u04cc\3\2\2\2\u04d0\u04cd\3\2\2\2\u04d0\u04ce\3")
        buf.write("\2\2\2\u04d0\u04cf\3\2\2\2\u04d1\u04da\3\2\2\2\u04d2\u04d3")
        buf.write("\f#\2\2\u04d3\u04d4\7]\2\2\u04d4\u04d9\5h\65\2\u04d5\u04d6")
        buf.write("\f\17\2\2\u04d6\u04d7\7p\2\2\u04d7\u04d9\5h\65\2\u04d8")
        buf.write("\u04d2\3\2\2\2\u04d8\u04d5\3\2\2\2\u04d9\u04dc\3\2\2\2")
        buf.write("\u04da\u04d8\3\2\2\2\u04da\u04db\3\2\2\2\u04dbe\3\2\2")
        buf.write("\2\u04dc\u04da\3\2\2\2\u04dd\u04de\t\4\2\2\u04deg\3\2")
        buf.write("\2\2\u04df\u04e0\t\4\2\2\u04e0i\3\2\2\2\u04e1\u04e2\t")
        buf.write("\4\2\2\u04e2k\3\2\2\2\u04e3\u04ee\7[\2\2\u04e4\u04ec\7")
        buf.write("\u009c\2\2\u04e5\u04e7\7\u009f\2\2\u04e6\u04e5\3\2\2\2")
        buf.write("\u04e7\u04ea\3\2\2\2\u04e8\u04e6\3\2\2\2\u04e8\u04e9\3")
        buf.write("\2\2\2\u04e9\u04ed\3\2\2\2\u04ea\u04e8\3\2\2\2\u04eb\u04ed")
        buf.write("\7\u00a0\2\2\u04ec\u04e8\3\2\2\2\u04ec\u04eb\3\2\2\2\u04ed")
        buf.write("\u04ef\3\2\2\2\u04ee\u04e4\3\2\2\2\u04ee\u04ef\3\2\2\2")
        buf.write("\u04ef\u04f0\3\2\2\2\u04f0\u04f4\7\u009d\2\2\u04f1\u04f3")
        buf.write("\7\u009f\2\2\u04f2\u04f1\3\2\2\2\u04f3\u04f6\3\2\2\2\u04f4")
        buf.write("\u04f2\3\2\2\2\u04f4\u04f5\3\2\2\2\u04f5\u04f7\3\2\2\2")
        buf.write("\u04f6\u04f4\3\2\2\2\u04f7\u04f8\7\u009e\2\2\u04f8m\3")
        buf.write("\2\2\2\u00a0{}\u0083\u0089\u008f\u0095\u009b\u00a4\u00ae")
        buf.write("\u00b8\u00c2\u00cb\u00d4\u00dd\u00e6\u00ef\u00f8\u0101")
        buf.write("\u010a\u0111\u0120\u0125\u012c\u0135\u013a\u0145\u0151")
        buf.write("\u0157\u0164\u016b\u0186\u018c\u018f\u019d\u01a2\u01a7")
        buf.write("\u01ae\u01b3\u01b8\u01bd\u01c3\u01c7\u01d7\u01dd\u01e1")
        buf.write("\u01e6\u01e9\u01ed\u01f2\u01f7\u01fb\u0201\u0205\u0209")
        buf.write("\u020c\u020e\u0212\u0217\u0221\u0226\u022b\u0232\u023e")
        buf.write("\u0243\u024e\u0254\u0258\u025d\u0262\u026b\u026e\u0271")
        buf.write("\u027d\u0284\u0289\u0295\u029a\u029f\u02a4\u02a9\u02b2")
        buf.write("\u02cd\u02d2\u02d8\u02e2\u02e5\u02ea\u02ef\u02f3\u02ff")
        buf.write("\u0317\u031f\u0326\u0329\u032c\u0331\u0336\u033b\u0345")
        buf.write("\u034a\u034f\u0353\u035c\u035f\u0366\u0369\u036c\u036f")
        buf.write("\u037b\u0382\u0389\u038e\u039a\u039f\u03a4\u03ab\u03ba")
        buf.write("\u03c8\u03cd\u03d2\u03d9\u03ea\u03ef\u03f8\u03fd\u0402")
        buf.write("\u0407\u0410\u0413\u041b\u042a\u0435\u043d\u044c\u045c")
        buf.write("\u045e\u0471\u0473\u047c\u0480\u048b\u048f\u0493\u0496")
        buf.write("\u04a3\u04a5\u04ac\u04ae\u04b5\u04ba\u04bc\u04d0\u04d8")
        buf.write("\u04da\u04e8\u04ec\u04ee\u04f4")
        return buf.getvalue()


class CnsMRParser ( Parser ):

    grammarFileName = "CnsMRParser.g4"

    atn = ATNDeserializer().deserialize(serializedATN())

    decisionsToDFA = [ DFA(ds, i) for i, ds in enumerate(atn.decisionToState) ]

    sharedContextCache = PredictionContextCache()

    literalNames = [ "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "'?'", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "','", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "'('", "')'", 
                     "':'", "'='", "'<'", "'>'", "'<='", "'>='", "'#'", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "<INVALID>", "<INVALID>", "<INVALID>", "<INVALID>", 
                     "'*'" ]

    symbolicNames = [ "<INVALID>", "Set", "End", "Noe", "Analysis", "Assign", 
                      "Asymptote", "Averaging", "Bgig", "Ceiling", "Classification", 
                      "CountViol", "Cv", "Den", "Distribute", "Ensemble", 
                      "Monomers", "Ncount", "Nrestraints", "Outd", "Partition", 
                      "Potential", "Predict", "Print", "Raverage", "Threshold", 
                      "Reset", "Rswitch", "Scale", "SoExponent", "SqConstant", 
                      "SqExponent", "SqOffset", "Taverage", "Temperature", 
                      "Noe_analysis", "Initialize", "Update", "Gamma", "Kappa", 
                      "Noe_avr_methods", "Noe_potential", "Cutoff", "Cuton", 
                      "From", "To", "Restraints", "Dihedral", "Nassign", 
                      "Print_any", "Plane", "Group", "Selection", "Weight", 
                      "Harmonic", "Exponent", "Normal", "Sanisotropy", "Coefficients", 
                      "ForceConstant", "Rdc_potential", "Coupling", "Coupling_potential", 
                      "Carbon", "Expectation", "PhiStep", "PsiStep", "Rcoil", 
                      "Zero", "Proton", "Observed", "Anisotropy", "Amides", 
                      "Carbons", "Nitrogens", "Oxygens", "RingAtoms", "AlphasAndAmides", 
                      "Error", "Conformation", "Compressed", "Phase", "Size", 
                      "Dimensions", "Danisotropy", "OneBond", "AngleDb", 
                      "DerivFlag", "Angle_dihedral", "Flags", "All", "Around", 
                      "Atom", "Attribute", "BondedTo", "ByGroup", "ByRes", 
                      "Chemical", "Fbox", "Hydrogen", "Id", "Known", "Name", 
                      "NONE", "Point", "Cut", "Previous", "Pseudo", "Residue", 
                      "Resname", "Saround", "SegIdentifier", "Sfbox", "Store_1", 
                      "Store_2", "Store_3", "Store_4", "Store_5", "Store_6", 
                      "Store_7", "Store_8", "Store_9", "Tag", "Tail", "Head", 
                      "Or_op", "And_op", "Not_op", "Comma", "Complex", "Integer", 
                      "Logical", "Real", "Double_quote_string", "Simple_name", 
                      "Simple_names", "Integers", "L_paren", "R_paren", 
                      "Colon", "Equ_op", "Lt_op", "Gt_op", "Leq_op", "Geq_op", 
                      "Neq_op", "SPACE", "COMMENT", "LINE_COMMENT", "SET_VARIABLE", 
                      "Abs", "Attr_properties", "Comparison_ops", "SPACE_ATTR", 
                      "Exclude", "Include", "End_flag", "Class_name", "Any_class", 
                      "SPACE_FLAG" ]

    RULE_cns_mr = 0
    RULE_distance_restraint = 1
    RULE_dihedral_angle_restraint = 2
    RULE_plane_restraint = 3
    RULE_harmonic_restraint = 4
    RULE_rdc_restraint = 5
    RULE_coupling_restraint = 6
    RULE_carbon_shift_restraint = 7
    RULE_proton_shift_restraint = 8
    RULE_conformation_db_restraint = 9
    RULE_diffusion_anisotropy_restraint = 10
    RULE_one_bond_coupling_restraint = 11
    RULE_angle_db_restraint = 12
    RULE_noe_statement = 13
    RULE_noe_assign = 14
    RULE_predict_statement = 15
    RULE_dihedral_statement = 16
    RULE_dihedral_assign = 17
    RULE_plane_statement = 18
    RULE_group_statement = 19
    RULE_harmonic_statement = 20
    RULE_sani_statement = 21
    RULE_sani_assign = 22
    RULE_coupling_statement = 23
    RULE_coup_assign = 24
    RULE_carbon_shift_statement = 25
    RULE_carbon_shift_assign = 26
    RULE_carbon_shift_rcoil = 27
    RULE_proton_shift_statement = 28
    RULE_observed = 29
    RULE_proton_shift_rcoil = 30
    RULE_proton_shift_anisotropy = 31
    RULE_proton_shift_amides = 32
    RULE_proton_shift_carbons = 33
    RULE_proton_shift_nitrogens = 34
    RULE_proton_shift_oxygens = 35
    RULE_proton_shift_ring_atoms = 36
    RULE_proton_shift_alphas_and_amides = 37
    RULE_conformation_statement = 38
    RULE_conf_assign = 39
    RULE_diffusion_statement = 40
    RULE_dani_assign = 41
    RULE_one_bond_coupling_statement = 42
    RULE_one_bond_assign = 43
    RULE_angle_db_statement = 44
    RULE_angle_db_assign = 45
    RULE_selection = 46
    RULE_selection_expression = 47
    RULE_term = 48
    RULE_factor = 49
    RULE_number = 50
    RULE_number_f = 51
    RULE_number_s = 52
    RULE_flag_statement = 53

    ruleNames =  [ "cns_mr", "distance_restraint", "dihedral_angle_restraint", 
                   "plane_restraint", "harmonic_restraint", "rdc_restraint", 
                   "coupling_restraint", "carbon_shift_restraint", "proton_shift_restraint", 
                   "conformation_db_restraint", "diffusion_anisotropy_restraint", 
                   "one_bond_coupling_restraint", "angle_db_restraint", 
                   "noe_statement", "noe_assign", "predict_statement", "dihedral_statement", 
                   "dihedral_assign", "plane_statement", "group_statement", 
                   "harmonic_statement", "sani_statement", "sani_assign", 
                   "coupling_statement", "coup_assign", "carbon_shift_statement", 
                   "carbon_shift_assign", "carbon_shift_rcoil", "proton_shift_statement", 
                   "observed", "proton_shift_rcoil", "proton_shift_anisotropy", 
                   "proton_shift_amides", "proton_shift_carbons", "proton_shift_nitrogens", 
                   "proton_shift_oxygens", "proton_shift_ring_atoms", "proton_shift_alphas_and_amides", 
                   "conformation_statement", "conf_assign", "diffusion_statement", 
                   "dani_assign", "one_bond_coupling_statement", "one_bond_assign", 
                   "angle_db_statement", "angle_db_assign", "selection", 
                   "selection_expression", "term", "factor", "number", "number_f", 
                   "number_s", "flag_statement" ]

    EOF = Token.EOF
    Set=1
    End=2
    Noe=3
    Analysis=4
    Assign=5
    Asymptote=6
    Averaging=7
    Bgig=8
    Ceiling=9
    Classification=10
    CountViol=11
    Cv=12
    Den=13
    Distribute=14
    Ensemble=15
    Monomers=16
    Ncount=17
    Nrestraints=18
    Outd=19
    Partition=20
    Potential=21
    Predict=22
    Print=23
    Raverage=24
    Threshold=25
    Reset=26
    Rswitch=27
    Scale=28
    SoExponent=29
    SqConstant=30
    SqExponent=31
    SqOffset=32
    Taverage=33
    Temperature=34
    Noe_analysis=35
    Initialize=36
    Update=37
    Gamma=38
    Kappa=39
    Noe_avr_methods=40
    Noe_potential=41
    Cutoff=42
    Cuton=43
    From=44
    To=45
    Restraints=46
    Dihedral=47
    Nassign=48
    Print_any=49
    Plane=50
    Group=51
    Selection=52
    Weight=53
    Harmonic=54
    Exponent=55
    Normal=56
    Sanisotropy=57
    Coefficients=58
    ForceConstant=59
    Rdc_potential=60
    Coupling=61
    Coupling_potential=62
    Carbon=63
    Expectation=64
    PhiStep=65
    PsiStep=66
    Rcoil=67
    Zero=68
    Proton=69
    Observed=70
    Anisotropy=71
    Amides=72
    Carbons=73
    Nitrogens=74
    Oxygens=75
    RingAtoms=76
    AlphasAndAmides=77
    Error=78
    Conformation=79
    Compressed=80
    Phase=81
    Size=82
    Dimensions=83
    Danisotropy=84
    OneBond=85
    AngleDb=86
    DerivFlag=87
    Angle_dihedral=88
    Flags=89
    All=90
    Around=91
    Atom=92
    Attribute=93
    BondedTo=94
    ByGroup=95
    ByRes=96
    Chemical=97
    Fbox=98
    Hydrogen=99
    Id=100
    Known=101
    Name=102
    NONE=103
    Point=104
    Cut=105
    Previous=106
    Pseudo=107
    Residue=108
    Resname=109
    Saround=110
    SegIdentifier=111
    Sfbox=112
    Store_1=113
    Store_2=114
    Store_3=115
    Store_4=116
    Store_5=117
    Store_6=118
    Store_7=119
    Store_8=120
    Store_9=121
    Tag=122
    Tail=123
    Head=124
    Or_op=125
    And_op=126
    Not_op=127
    Comma=128
    Complex=129
    Integer=130
    Logical=131
    Real=132
    Double_quote_string=133
    Simple_name=134
    Simple_names=135
    Integers=136
    L_paren=137
    R_paren=138
    Colon=139
    Equ_op=140
    Lt_op=141
    Gt_op=142
    Leq_op=143
    Geq_op=144
    Neq_op=145
    SPACE=146
    COMMENT=147
    LINE_COMMENT=148
    SET_VARIABLE=149
    Abs=150
    Attr_properties=151
    Comparison_ops=152
    SPACE_ATTR=153
    Exclude=154
    Include=155
    End_flag=156
    Class_name=157
    Any_class=158
    SPACE_FLAG=159

    def __init__(self, input:TokenStream, output:TextIO = sys.stdout):
        super().__init__(input, output)
        self.checkVersion("4.9")
        self._interp = ParserATNSimulator(self, self.atn, self.decisionsToDFA, self.sharedContextCache)
        self._predicates = None




    class Cns_mrContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def EOF(self):
            return self.getToken(CnsMRParser.EOF, 0)

        def distance_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Distance_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Distance_restraintContext,i)


        def dihedral_angle_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_angle_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_angle_restraintContext,i)


        def plane_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Plane_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Plane_restraintContext,i)


        def harmonic_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Harmonic_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Harmonic_restraintContext,i)


        def rdc_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Rdc_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Rdc_restraintContext,i)


        def coupling_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coupling_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coupling_restraintContext,i)


        def carbon_shift_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Carbon_shift_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Carbon_shift_restraintContext,i)


        def proton_shift_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_restraintContext,i)


        def conformation_db_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Conformation_db_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Conformation_db_restraintContext,i)


        def diffusion_anisotropy_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Diffusion_anisotropy_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Diffusion_anisotropy_restraintContext,i)


        def one_bond_coupling_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.One_bond_coupling_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.One_bond_coupling_restraintContext,i)


        def angle_db_restraint(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Angle_db_restraintContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Angle_db_restraintContext,i)


        def flag_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Flag_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Flag_statementContext,i)


        def noe_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Noe_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Noe_assignContext,i)


        def dihedral_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_assignContext,i)


        def sani_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Sani_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Sani_assignContext,i)


        def plane_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Plane_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Plane_statementContext,i)


        def coup_assign(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coup_assignContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coup_assignContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_cns_mr

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCns_mr" ):
                listener.enterCns_mr(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCns_mr" ):
                listener.exitCns_mr(self)




    def cns_mr(self):

        localctx = CnsMRParser.Cns_mrContext(self, self._ctx, self.state)
        self.enterRule(localctx, 0, self.RULE_cns_mr)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 123
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Noe) | (1 << CnsMRParser.Restraints) | (1 << CnsMRParser.Sanisotropy) | (1 << CnsMRParser.Coupling) | (1 << CnsMRParser.Carbon))) != 0) or ((((_la - 69)) & ~0x3f) == 0 and ((1 << (_la - 69)) & ((1 << (CnsMRParser.Proton - 69)) | (1 << (CnsMRParser.Conformation - 69)) | (1 << (CnsMRParser.Danisotropy - 69)) | (1 << (CnsMRParser.OneBond - 69)) | (1 << (CnsMRParser.AngleDb - 69)) | (1 << (CnsMRParser.Flags - 69)))) != 0):
                self.state = 121
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,0,self._ctx)
                if la_ == 1:
                    self.state = 108
                    self.distance_restraint()
                    pass

                elif la_ == 2:
                    self.state = 109
                    self.dihedral_angle_restraint()
                    pass

                elif la_ == 3:
                    self.state = 110
                    self.plane_restraint()
                    pass

                elif la_ == 4:
                    self.state = 111
                    self.harmonic_restraint()
                    pass

                elif la_ == 5:
                    self.state = 112
                    self.rdc_restraint()
                    pass

                elif la_ == 6:
                    self.state = 113
                    self.coupling_restraint()
                    pass

                elif la_ == 7:
                    self.state = 114
                    self.carbon_shift_restraint()
                    pass

                elif la_ == 8:
                    self.state = 115
                    self.proton_shift_restraint()
                    pass

                elif la_ == 9:
                    self.state = 116
                    self.conformation_db_restraint()
                    pass

                elif la_ == 10:
                    self.state = 117
                    self.diffusion_anisotropy_restraint()
                    pass

                elif la_ == 11:
                    self.state = 118
                    self.one_bond_coupling_restraint()
                    pass

                elif la_ == 12:
                    self.state = 119
                    self.angle_db_restraint()
                    pass

                elif la_ == 13:
                    self.state = 120
                    self.flag_statement()
                    pass


                self.state = 125
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 129
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,2,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 126
                    self.noe_assign() 
                self.state = 131
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,2,self._ctx)

            self.state = 135
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,3,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 132
                    self.dihedral_assign() 
                self.state = 137
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,3,self._ctx)

            self.state = 141
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,4,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 138
                    self.sani_assign() 
                self.state = 143
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,4,self._ctx)

            self.state = 147
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Initialize) | (1 << CnsMRParser.Print_any) | (1 << CnsMRParser.Group))) != 0):
                self.state = 144
                self.plane_statement()
                self.state = 149
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 153
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Assign:
                self.state = 150
                self.coup_assign()
                self.state = 155
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 156
            self.match(CnsMRParser.EOF)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Distance_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Noe(self):
            return self.getToken(CnsMRParser.Noe, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def noe_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Noe_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Noe_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_distance_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDistance_restraint" ):
                listener.enterDistance_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDistance_restraint" ):
                listener.exitDistance_restraint(self)




    def distance_restraint(self):

        localctx = CnsMRParser.Distance_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 2, self.RULE_distance_restraint)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 158
            self.match(CnsMRParser.Noe)
            self.state = 162
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,7,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    self.state = 159
                    self.noe_statement() 
                self.state = 164
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,7,self._ctx)

            self.state = 165
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_angle_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Dihedral(self):
            return self.getToken(CnsMRParser.Dihedral, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def dihedral_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Dihedral_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Dihedral_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_angle_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_angle_restraint" ):
                listener.enterDihedral_angle_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_angle_restraint" ):
                listener.exitDihedral_angle_restraint(self)




    def dihedral_angle_restraint(self):

        localctx = CnsMRParser.Dihedral_angle_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 4, self.RULE_dihedral_angle_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 167
            self.match(CnsMRParser.Restraints)
            self.state = 168
            self.match(CnsMRParser.Dihedral)
            self.state = 172
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Cv) | (1 << CnsMRParser.Partition) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.Scale) | (1 << CnsMRParser.Nassign) | (1 << CnsMRParser.Print_any))) != 0):
                self.state = 169
                self.dihedral_statement()
                self.state = 174
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 175
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Plane_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Plane(self):
            return self.getToken(CnsMRParser.Plane, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def plane_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Plane_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Plane_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_plane_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlane_restraint" ):
                listener.enterPlane_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlane_restraint" ):
                listener.exitPlane_restraint(self)




    def plane_restraint(self):

        localctx = CnsMRParser.Plane_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 6, self.RULE_plane_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 177
            self.match(CnsMRParser.Restraints)
            self.state = 178
            self.match(CnsMRParser.Plane)
            self.state = 182
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Initialize) | (1 << CnsMRParser.Print_any) | (1 << CnsMRParser.Group))) != 0):
                self.state = 179
                self.plane_statement()
                self.state = 184
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 185
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Harmonic_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Restraints(self):
            return self.getToken(CnsMRParser.Restraints, 0)

        def Harmonic(self):
            return self.getToken(CnsMRParser.Harmonic, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def harmonic_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Harmonic_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Harmonic_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_harmonic_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHarmonic_restraint" ):
                listener.enterHarmonic_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHarmonic_restraint" ):
                listener.exitHarmonic_restraint(self)




    def harmonic_restraint(self):

        localctx = CnsMRParser.Harmonic_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 8, self.RULE_harmonic_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 187
            self.match(CnsMRParser.Restraints)
            self.state = 188
            self.match(CnsMRParser.Harmonic)
            self.state = 192
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Exponent or _la==CnsMRParser.Normal:
                self.state = 189
                self.harmonic_statement()
                self.state = 194
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 195
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Rdc_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Sanisotropy(self):
            return self.getToken(CnsMRParser.Sanisotropy, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def sani_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Sani_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Sani_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_rdc_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterRdc_restraint" ):
                listener.enterRdc_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitRdc_restraint" ):
                listener.exitRdc_restraint(self)




    def rdc_restraint(self):

        localctx = CnsMRParser.Rdc_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 10, self.RULE_rdc_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 197
            self.match(CnsMRParser.Sanisotropy)
            self.state = 201
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.Coefficients) | (1 << CnsMRParser.ForceConstant))) != 0):
                self.state = 198
                self.sani_statement()
                self.state = 203
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 204
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coupling_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Coupling(self):
            return self.getToken(CnsMRParser.Coupling, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def coupling_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Coupling_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Coupling_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_coupling_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoupling_restraint" ):
                listener.enterCoupling_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoupling_restraint" ):
                listener.exitCoupling_restraint(self)




    def coupling_restraint(self):

        localctx = CnsMRParser.Coupling_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 12, self.RULE_coupling_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 206
            self.match(CnsMRParser.Coupling)
            self.state = 210
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Cv) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Partition) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.Coefficients) | (1 << CnsMRParser.ForceConstant))) != 0):
                self.state = 207
                self.coupling_statement()
                self.state = 212
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 213
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Carbon(self):
            return self.getToken(CnsMRParser.Carbon, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def carbon_shift_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Carbon_shift_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Carbon_shift_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_restraint" ):
                listener.enterCarbon_shift_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_restraint" ):
                listener.exitCarbon_shift_restraint(self)




    def carbon_shift_restraint(self):

        localctx = CnsMRParser.Carbon_shift_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 14, self.RULE_carbon_shift_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 215
            self.match(CnsMRParser.Carbon)
            self.state = 219
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while ((((_la - 5)) & ~0x3f) == 0 and ((1 << (_la - 5)) & ((1 << (CnsMRParser.Assign - 5)) | (1 << (CnsMRParser.Classification - 5)) | (1 << (CnsMRParser.Nrestraints - 5)) | (1 << (CnsMRParser.Potential - 5)) | (1 << (CnsMRParser.Print - 5)) | (1 << (CnsMRParser.Reset - 5)) | (1 << (CnsMRParser.ForceConstant - 5)) | (1 << (CnsMRParser.Expectation - 5)) | (1 << (CnsMRParser.PhiStep - 5)) | (1 << (CnsMRParser.PsiStep - 5)) | (1 << (CnsMRParser.Rcoil - 5)) | (1 << (CnsMRParser.Zero - 5)))) != 0):
                self.state = 216
                self.carbon_shift_statement()
                self.state = 221
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 222
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Proton(self):
            return self.getToken(CnsMRParser.Proton, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def proton_shift_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Proton_shift_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Proton_shift_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_restraint" ):
                listener.enterProton_shift_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_restraint" ):
                listener.exitProton_shift_restraint(self)




    def proton_shift_restraint(self):

        localctx = CnsMRParser.Proton_shift_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 16, self.RULE_proton_shift_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 224
            self.match(CnsMRParser.Proton)
            self.state = 228
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Classification) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.ForceConstant))) != 0) or ((((_la - 67)) & ~0x3f) == 0 and ((1 << (_la - 67)) & ((1 << (CnsMRParser.Rcoil - 67)) | (1 << (CnsMRParser.Observed - 67)) | (1 << (CnsMRParser.Anisotropy - 67)) | (1 << (CnsMRParser.Amides - 67)) | (1 << (CnsMRParser.Carbons - 67)) | (1 << (CnsMRParser.Nitrogens - 67)) | (1 << (CnsMRParser.Oxygens - 67)) | (1 << (CnsMRParser.RingAtoms - 67)) | (1 << (CnsMRParser.AlphasAndAmides - 67)) | (1 << (CnsMRParser.Error - 67)))) != 0):
                self.state = 225
                self.proton_shift_statement()
                self.state = 230
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 231
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conformation_db_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Conformation(self):
            return self.getToken(CnsMRParser.Conformation, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def conformation_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Conformation_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Conformation_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_conformation_db_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConformation_db_restraint" ):
                listener.enterConformation_db_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConformation_db_restraint" ):
                listener.exitConformation_db_restraint(self)




    def conformation_db_restraint(self):

        localctx = CnsMRParser.Conformation_db_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 18, self.RULE_conformation_db_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 233
            self.match(CnsMRParser.Conformation)
            self.state = 237
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.ForceConstant))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (CnsMRParser.Expectation - 64)) | (1 << (CnsMRParser.Zero - 64)) | (1 << (CnsMRParser.Error - 64)) | (1 << (CnsMRParser.Compressed - 64)) | (1 << (CnsMRParser.Phase - 64)) | (1 << (CnsMRParser.Size - 64)))) != 0):
                self.state = 234
                self.conformation_statement()
                self.state = 239
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 240
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Diffusion_anisotropy_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Danisotropy(self):
            return self.getToken(CnsMRParser.Danisotropy, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def diffusion_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Diffusion_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Diffusion_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_diffusion_anisotropy_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDiffusion_anisotropy_restraint" ):
                listener.enterDiffusion_anisotropy_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDiffusion_anisotropy_restraint" ):
                listener.exitDiffusion_anisotropy_restraint(self)




    def diffusion_anisotropy_restraint(self):

        localctx = CnsMRParser.Diffusion_anisotropy_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 20, self.RULE_diffusion_anisotropy_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 242
            self.match(CnsMRParser.Danisotropy)
            self.state = 246
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.Coefficients) | (1 << CnsMRParser.ForceConstant))) != 0):
                self.state = 243
                self.diffusion_statement()
                self.state = 248
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 249
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_coupling_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def OneBond(self):
            return self.getToken(CnsMRParser.OneBond, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def one_bond_coupling_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.One_bond_coupling_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.One_bond_coupling_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_coupling_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_coupling_restraint" ):
                listener.enterOne_bond_coupling_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_coupling_restraint" ):
                listener.exitOne_bond_coupling_restraint(self)




    def one_bond_coupling_restraint(self):

        localctx = CnsMRParser.One_bond_coupling_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 22, self.RULE_one_bond_coupling_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 251
            self.match(CnsMRParser.OneBond)
            self.state = 255
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.Coefficients) | (1 << CnsMRParser.ForceConstant))) != 0):
                self.state = 252
                self.one_bond_coupling_statement()
                self.state = 257
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 258
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_restraintContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AngleDb(self):
            return self.getToken(CnsMRParser.AngleDb, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def angle_db_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Angle_db_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Angle_db_statementContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_restraint

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_restraint" ):
                listener.enterAngle_db_restraint(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_restraint" ):
                listener.exitAngle_db_restraint(self)




    def angle_db_restraint(self):

        localctx = CnsMRParser.Angle_db_restraintContext(self, self._ctx, self.state)
        self.enterRule(localctx, 24, self.RULE_angle_db_restraint)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 260
            self.match(CnsMRParser.AngleDb)
            self.state = 264
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while (((_la) & ~0x3f) == 0 and ((1 << _la) & ((1 << CnsMRParser.Assign) | (1 << CnsMRParser.Classification) | (1 << CnsMRParser.Nrestraints) | (1 << CnsMRParser.Potential) | (1 << CnsMRParser.Print) | (1 << CnsMRParser.Reset) | (1 << CnsMRParser.ForceConstant))) != 0) or ((((_la - 64)) & ~0x3f) == 0 and ((1 << (_la - 64)) & ((1 << (CnsMRParser.Expectation - 64)) | (1 << (CnsMRParser.Zero - 64)) | (1 << (CnsMRParser.Error - 64)) | (1 << (CnsMRParser.Size - 64)) | (1 << (CnsMRParser.DerivFlag - 64)))) != 0):
                self.state = 261
                self.angle_db_statement()
                self.state = 266
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 267
            self.match(CnsMRParser.End)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Analysis(self):
            return self.getToken(CnsMRParser.Analysis, 0)

        def Noe_analysis(self):
            return self.getToken(CnsMRParser.Noe_analysis, 0)

        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def noe_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Noe_assignContext,0)


        def Asymptote(self):
            return self.getToken(CnsMRParser.Asymptote, 0)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def Averaging(self):
            return self.getToken(CnsMRParser.Averaging, 0)

        def Noe_avr_methods(self):
            return self.getToken(CnsMRParser.Noe_avr_methods, 0)

        def Bgig(self):
            return self.getToken(CnsMRParser.Bgig, 0)

        def Ceiling(self):
            return self.getToken(CnsMRParser.Ceiling, 0)

        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def CountViol(self):
            return self.getToken(CnsMRParser.CountViol, 0)

        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Den(self):
            return self.getToken(CnsMRParser.Den, 0)

        def Initialize(self):
            return self.getToken(CnsMRParser.Initialize, 0)

        def Update(self):
            return self.getToken(CnsMRParser.Update, 0)

        def Gamma(self):
            return self.getToken(CnsMRParser.Gamma, 0)

        def Kappa(self):
            return self.getToken(CnsMRParser.Kappa, 0)

        def Distribute(self):
            return self.getToken(CnsMRParser.Distribute, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def Ensemble(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Ensemble)
            else:
                return self.getToken(CnsMRParser.Ensemble, i)

        def Monomers(self):
            return self.getToken(CnsMRParser.Monomers, 0)

        def Ncount(self):
            return self.getToken(CnsMRParser.Ncount, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Outd(self):
            return self.getToken(CnsMRParser.Outd, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Noe_potential(self):
            return self.getToken(CnsMRParser.Noe_potential, 0)

        def Predict(self):
            return self.getToken(CnsMRParser.Predict, 0)

        def predict_statement(self):
            return self.getTypedRuleContext(CnsMRParser.Predict_statementContext,0)


        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Raverage(self):
            return self.getToken(CnsMRParser.Raverage, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Rswitch(self):
            return self.getToken(CnsMRParser.Rswitch, 0)

        def Scale(self):
            return self.getToken(CnsMRParser.Scale, 0)

        def SoExponent(self):
            return self.getToken(CnsMRParser.SoExponent, 0)

        def SqConstant(self):
            return self.getToken(CnsMRParser.SqConstant, 0)

        def SqExponent(self):
            return self.getToken(CnsMRParser.SqExponent, 0)

        def SqOffset(self):
            return self.getToken(CnsMRParser.SqOffset, 0)

        def Taverage(self):
            return self.getToken(CnsMRParser.Taverage, 0)

        def Temperature(self):
            return self.getToken(CnsMRParser.Temperature, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_noe_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_statement" ):
                listener.enterNoe_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_statement" ):
                listener.exitNoe_statement(self)




    def noe_statement(self):

        localctx = CnsMRParser.Noe_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 26, self.RULE_noe_statement)
        self._la = 0 # Token type
        try:
            self.state = 397
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,32,self._ctx)
            if la_ == 1:
                self.enterOuterAlt(localctx, 1)
                self.state = 269
                self.match(CnsMRParser.Analysis)
                self.state = 271
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 270
                    self.match(CnsMRParser.Equ_op)


                self.state = 273
                self.match(CnsMRParser.Noe_analysis)
                pass

            elif la_ == 2:
                self.enterOuterAlt(localctx, 2)
                self.state = 274
                self.noe_assign()
                pass

            elif la_ == 3:
                self.enterOuterAlt(localctx, 3)
                self.state = 275
                self.match(CnsMRParser.Asymptote)
                self.state = 276
                self.match(CnsMRParser.Simple_name)
                self.state = 277
                self.number_s()
                pass

            elif la_ == 4:
                self.enterOuterAlt(localctx, 4)
                self.state = 278
                self.match(CnsMRParser.Averaging)
                self.state = 279
                self.match(CnsMRParser.Simple_name)
                self.state = 280
                self.match(CnsMRParser.Noe_avr_methods)
                pass

            elif la_ == 5:
                self.enterOuterAlt(localctx, 5)
                self.state = 281
                self.match(CnsMRParser.Bgig)
                self.state = 282
                self.match(CnsMRParser.Simple_name)
                self.state = 283
                self.number_s()
                pass

            elif la_ == 6:
                self.enterOuterAlt(localctx, 6)
                self.state = 284
                self.match(CnsMRParser.Ceiling)
                self.state = 286
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 285
                    self.match(CnsMRParser.Equ_op)


                self.state = 288
                self.number_s()
                pass

            elif la_ == 7:
                self.enterOuterAlt(localctx, 7)
                self.state = 289
                self.match(CnsMRParser.Classification)
                self.state = 291
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 290
                    self.match(CnsMRParser.Equ_op)


                self.state = 293
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 8:
                self.enterOuterAlt(localctx, 8)
                self.state = 294
                self.match(CnsMRParser.CountViol)
                self.state = 295
                self.match(CnsMRParser.Simple_name)
                pass

            elif la_ == 9:
                self.enterOuterAlt(localctx, 9)
                self.state = 296
                self.match(CnsMRParser.Cv)
                self.state = 298
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 297
                    self.match(CnsMRParser.Equ_op)


                self.state = 300
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 10:
                self.enterOuterAlt(localctx, 10)
                self.state = 301
                self.match(CnsMRParser.Den)
                self.state = 302
                self.match(CnsMRParser.Initialize)
                pass

            elif la_ == 11:
                self.enterOuterAlt(localctx, 11)
                self.state = 303
                self.match(CnsMRParser.Den)
                self.state = 304
                self.match(CnsMRParser.Update)
                self.state = 305
                self.match(CnsMRParser.Gamma)
                self.state = 307
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 306
                    self.match(CnsMRParser.Equ_op)


                self.state = 309
                self.number_s()
                self.state = 310
                self.match(CnsMRParser.Kappa)
                self.state = 312
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 311
                    self.match(CnsMRParser.Equ_op)


                self.state = 314
                self.number_s()
                pass

            elif la_ == 12:
                self.enterOuterAlt(localctx, 12)
                self.state = 316
                self.match(CnsMRParser.Distribute)
                self.state = 317
                self.match(CnsMRParser.Simple_name)
                self.state = 318
                self.match(CnsMRParser.Simple_name)
                self.state = 319
                self.number_s()
                pass

            elif la_ == 13:
                self.enterOuterAlt(localctx, 13)
                self.state = 323
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,25,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 320
                        self.match(CnsMRParser.Ensemble) 
                    self.state = 325
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,25,self._ctx)

                self.state = 326
                self.match(CnsMRParser.End)
                pass

            elif la_ == 14:
                self.enterOuterAlt(localctx, 14)
                self.state = 327
                self.match(CnsMRParser.Monomers)
                self.state = 328
                self.match(CnsMRParser.Simple_name)
                self.state = 329
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 15:
                self.enterOuterAlt(localctx, 15)
                self.state = 330
                self.match(CnsMRParser.Ncount)
                self.state = 331
                self.match(CnsMRParser.Simple_name)
                self.state = 332
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 16:
                self.enterOuterAlt(localctx, 16)
                self.state = 333
                self.match(CnsMRParser.Nrestraints)
                self.state = 335
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 334
                    self.match(CnsMRParser.Equ_op)


                self.state = 337
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 17:
                self.enterOuterAlt(localctx, 17)
                self.state = 338
                self.match(CnsMRParser.Outd)
                pass

            elif la_ == 18:
                self.enterOuterAlt(localctx, 18)
                self.state = 339
                self.match(CnsMRParser.Partition)
                self.state = 341
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 340
                    self.match(CnsMRParser.Equ_op)


                self.state = 343
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 19:
                self.enterOuterAlt(localctx, 19)
                self.state = 344
                self.match(CnsMRParser.Potential)
                self.state = 345
                self.match(CnsMRParser.Simple_name)
                self.state = 346
                self.match(CnsMRParser.Noe_potential)
                pass

            elif la_ == 20:
                self.enterOuterAlt(localctx, 20)
                self.state = 347
                self.match(CnsMRParser.Predict)
                self.state = 348
                self.predict_statement()
                self.state = 349
                self.match(CnsMRParser.End)
                pass

            elif la_ == 21:
                self.enterOuterAlt(localctx, 21)
                self.state = 351
                self.match(CnsMRParser.Print)
                self.state = 352
                self.match(CnsMRParser.Threshold)
                self.state = 354
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 353
                    self.match(CnsMRParser.Equ_op)


                self.state = 356
                self.number_s()
                pass

            elif la_ == 22:
                self.enterOuterAlt(localctx, 22)
                self.state = 357
                self.match(CnsMRParser.Raverage)
                self.state = 361
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,29,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 358
                        self.match(CnsMRParser.Simple_name) 
                    self.state = 363
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,29,self._ctx)

                self.state = 364
                self.match(CnsMRParser.End)
                pass

            elif la_ == 23:
                self.enterOuterAlt(localctx, 23)
                self.state = 365
                self.match(CnsMRParser.Reset)
                pass

            elif la_ == 24:
                self.enterOuterAlt(localctx, 24)
                self.state = 366
                self.match(CnsMRParser.Rswitch)
                self.state = 367
                self.match(CnsMRParser.Simple_name)
                self.state = 368
                self.number_s()
                pass

            elif la_ == 25:
                self.enterOuterAlt(localctx, 25)
                self.state = 369
                self.match(CnsMRParser.Scale)
                self.state = 370
                self.match(CnsMRParser.Simple_name)
                self.state = 371
                self.number_s()
                pass

            elif la_ == 26:
                self.enterOuterAlt(localctx, 26)
                self.state = 372
                self.match(CnsMRParser.SoExponent)
                self.state = 373
                self.match(CnsMRParser.Simple_name)
                self.state = 374
                self.number_s()
                pass

            elif la_ == 27:
                self.enterOuterAlt(localctx, 27)
                self.state = 375
                self.match(CnsMRParser.SqConstant)
                self.state = 376
                self.match(CnsMRParser.Simple_name)
                self.state = 377
                self.number_s()
                pass

            elif la_ == 28:
                self.enterOuterAlt(localctx, 28)
                self.state = 378
                self.match(CnsMRParser.SqExponent)
                self.state = 379
                self.match(CnsMRParser.Simple_name)
                self.state = 380
                self.number_s()
                pass

            elif la_ == 29:
                self.enterOuterAlt(localctx, 29)
                self.state = 381
                self.match(CnsMRParser.SqOffset)
                self.state = 382
                self.match(CnsMRParser.Simple_name)
                self.state = 383
                self.number_s()
                pass

            elif la_ == 30:
                self.enterOuterAlt(localctx, 30)
                self.state = 384
                self.match(CnsMRParser.Taverage)
                self.state = 388
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,30,self._ctx)
                while _alt!=1 and _alt!=ATN.INVALID_ALT_NUMBER:
                    if _alt==1+1:
                        self.state = 385
                        self.match(CnsMRParser.Simple_name) 
                    self.state = 390
                    self._errHandler.sync(self)
                    _alt = self._interp.adaptivePredict(self._input,30,self._ctx)

                self.state = 391
                self.match(CnsMRParser.End)
                pass

            elif la_ == 31:
                self.enterOuterAlt(localctx, 31)
                self.state = 392
                self.match(CnsMRParser.Temperature)
                self.state = 394
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 393
                    self.match(CnsMRParser.Equ_op)


                self.state = 396
                self.number_s()
                pass


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Noe_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def Or_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Or_op)
            else:
                return self.getToken(CnsMRParser.Or_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_noe_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNoe_assign" ):
                listener.enterNoe_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNoe_assign" ):
                listener.exitNoe_assign(self)




    def noe_assign(self):

        localctx = CnsMRParser.Noe_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 28, self.RULE_noe_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 399
            self.match(CnsMRParser.Assign)
            self.state = 400
            self.selection()
            self.state = 401
            self.selection()
            self.state = 402
            self.number()
            self.state = 403
            self.number()
            self.state = 404
            self.number()
            self.state = 411
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Or_op:
                self.state = 405
                self.match(CnsMRParser.Or_op)
                self.state = 406
                self.selection()
                self.state = 407
                self.selection()
                self.state = 413
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Predict_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Cutoff(self):
            return self.getToken(CnsMRParser.Cutoff, 0)

        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Cuton(self):
            return self.getToken(CnsMRParser.Cuton, 0)

        def From(self):
            return self.getToken(CnsMRParser.From, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def To(self):
            return self.getToken(CnsMRParser.To, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_predict_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPredict_statement" ):
                listener.enterPredict_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPredict_statement" ):
                listener.exitPredict_statement(self)




    def predict_statement(self):

        localctx = CnsMRParser.Predict_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 30, self.RULE_predict_statement)
        self._la = 0 # Token type
        try:
            self.state = 428
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Cutoff]:
                self.enterOuterAlt(localctx, 1)
                self.state = 414
                self.match(CnsMRParser.Cutoff)
                self.state = 416
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 415
                    self.match(CnsMRParser.Equ_op)


                self.state = 418
                self.number_s()
                pass
            elif token in [CnsMRParser.Cuton]:
                self.enterOuterAlt(localctx, 2)
                self.state = 419
                self.match(CnsMRParser.Cuton)
                self.state = 421
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 420
                    self.match(CnsMRParser.Equ_op)


                self.state = 423
                self.number_s()
                pass
            elif token in [CnsMRParser.From]:
                self.enterOuterAlt(localctx, 3)
                self.state = 424
                self.match(CnsMRParser.From)
                self.state = 425
                self.selection()
                pass
            elif token in [CnsMRParser.To]:
                self.enterOuterAlt(localctx, 4)
                self.state = 426
                self.match(CnsMRParser.To)
                self.state = 427
                self.selection()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dihedral_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Dihedral_assignContext,0)


        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Nassign(self):
            return self.getToken(CnsMRParser.Nassign, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Scale(self):
            return self.getToken(CnsMRParser.Scale, 0)

        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def Print_any(self):
            return self.getToken(CnsMRParser.Print_any, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_statement" ):
                listener.enterDihedral_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_statement" ):
                listener.exitDihedral_statement(self)




    def dihedral_statement(self):

        localctx = CnsMRParser.Dihedral_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 32, self.RULE_dihedral_statement)
        self._la = 0 # Token type
        try:
            self.state = 453
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 430
                self.dihedral_assign()
                pass
            elif token in [CnsMRParser.Cv]:
                self.enterOuterAlt(localctx, 2)
                self.state = 431
                self.match(CnsMRParser.Cv)
                self.state = 433
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 432
                    self.match(CnsMRParser.Equ_op)


                self.state = 435
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Nassign]:
                self.enterOuterAlt(localctx, 3)
                self.state = 436
                self.match(CnsMRParser.Nassign)
                self.state = 438
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 437
                    self.match(CnsMRParser.Equ_op)


                self.state = 440
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Partition]:
                self.enterOuterAlt(localctx, 4)
                self.state = 441
                self.match(CnsMRParser.Partition)
                self.state = 443
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 442
                    self.match(CnsMRParser.Equ_op)


                self.state = 445
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 5)
                self.state = 446
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Scale]:
                self.enterOuterAlt(localctx, 6)
                self.state = 447
                self.match(CnsMRParser.Scale)
                self.state = 449
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 448
                    self.match(CnsMRParser.Equ_op)


                self.state = 451
                self.number_s()
                pass
            elif token in [CnsMRParser.Print_any]:
                self.enterOuterAlt(localctx, 7)
                self.state = 452
                self.match(CnsMRParser.Print_any)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dihedral_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_dihedral_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDihedral_assign" ):
                listener.enterDihedral_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDihedral_assign" ):
                listener.exitDihedral_assign(self)




    def dihedral_assign(self):

        localctx = CnsMRParser.Dihedral_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 34, self.RULE_dihedral_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 455
            self.match(CnsMRParser.Assign)
            self.state = 456
            self.selection()
            self.state = 457
            self.selection()
            self.state = 458
            self.selection()
            self.state = 459
            self.selection()
            self.state = 460
            self.number()
            self.state = 461
            self.number()
            self.state = 462
            self.number()
            self.state = 463
            self.match(CnsMRParser.Integer)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Plane_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Group(self):
            return self.getToken(CnsMRParser.Group, 0)

        def End(self):
            return self.getToken(CnsMRParser.End, 0)

        def group_statement(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Group_statementContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Group_statementContext,i)


        def Initialize(self):
            return self.getToken(CnsMRParser.Initialize, 0)

        def Print_any(self):
            return self.getToken(CnsMRParser.Print_any, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_plane_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterPlane_statement" ):
                listener.enterPlane_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitPlane_statement" ):
                listener.exitPlane_statement(self)




    def plane_statement(self):

        localctx = CnsMRParser.Plane_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 36, self.RULE_plane_statement)
        self._la = 0 # Token type
        try:
            self.state = 475
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Group]:
                self.enterOuterAlt(localctx, 1)
                self.state = 465
                self.match(CnsMRParser.Group)
                self.state = 469
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                while _la==CnsMRParser.Selection or _la==CnsMRParser.Weight:
                    self.state = 466
                    self.group_statement()
                    self.state = 471
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)

                self.state = 472
                self.match(CnsMRParser.End)
                pass
            elif token in [CnsMRParser.Initialize]:
                self.enterOuterAlt(localctx, 2)
                self.state = 473
                self.match(CnsMRParser.Initialize)
                pass
            elif token in [CnsMRParser.Print_any]:
                self.enterOuterAlt(localctx, 3)
                self.state = 474
                self.match(CnsMRParser.Print_any)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Group_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Selection(self):
            return self.getToken(CnsMRParser.Selection, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Weight(self):
            return self.getToken(CnsMRParser.Weight, 0)

        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_group_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterGroup_statement" ):
                listener.enterGroup_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitGroup_statement" ):
                listener.exitGroup_statement(self)




    def group_statement(self):

        localctx = CnsMRParser.Group_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 38, self.RULE_group_statement)
        self._la = 0 # Token type
        try:
            self.state = 487
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Selection]:
                self.enterOuterAlt(localctx, 1)
                self.state = 477
                self.match(CnsMRParser.Selection)
                self.state = 479
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 478
                    self.match(CnsMRParser.Equ_op)


                self.state = 481
                self.selection()
                pass
            elif token in [CnsMRParser.Weight]:
                self.enterOuterAlt(localctx, 2)
                self.state = 482
                self.match(CnsMRParser.Weight)
                self.state = 484
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 483
                    self.match(CnsMRParser.Equ_op)


                self.state = 486
                self.number_s()
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Harmonic_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Exponent(self):
            return self.getToken(CnsMRParser.Exponent, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def Normal(self):
            return self.getToken(CnsMRParser.Normal, 0)

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def Tail(self):
            return self.getToken(CnsMRParser.Tail, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Comma(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Comma)
            else:
                return self.getToken(CnsMRParser.Comma, i)

        def Head(self):
            return self.getToken(CnsMRParser.Head, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_harmonic_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterHarmonic_statement" ):
                listener.enterHarmonic_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitHarmonic_statement" ):
                listener.exitHarmonic_statement(self)




    def harmonic_statement(self):

        localctx = CnsMRParser.Harmonic_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 40, self.RULE_harmonic_statement)
        self._la = 0 # Token type
        try:
            self.state = 528
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Exponent]:
                self.enterOuterAlt(localctx, 1)
                self.state = 489
                self.match(CnsMRParser.Exponent)
                self.state = 491
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 490
                    self.match(CnsMRParser.Equ_op)


                self.state = 493
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Normal]:
                self.enterOuterAlt(localctx, 2)
                self.state = 494
                self.match(CnsMRParser.Normal)
                self.state = 496
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 495
                    self.match(CnsMRParser.Equ_op)


                self.state = 498
                self.match(CnsMRParser.L_paren)
                self.state = 524
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Integer, CnsMRParser.Real]:
                    self.state = 499
                    self.number_s()
                    self.state = 501
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 500
                        self.match(CnsMRParser.Comma)


                    self.state = 503
                    self.number_s()
                    self.state = 505
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 504
                        self.match(CnsMRParser.Comma)


                    self.state = 507
                    self.number_s()
                    pass
                elif token in [CnsMRParser.Tail]:
                    self.state = 509
                    self.match(CnsMRParser.Tail)
                    self.state = 511
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 510
                        self.match(CnsMRParser.Equ_op)


                    self.state = 513
                    self.selection()
                    self.state = 515
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Comma:
                        self.state = 514
                        self.match(CnsMRParser.Comma)


                    self.state = 522
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Head:
                        self.state = 517
                        self.match(CnsMRParser.Head)
                        self.state = 519
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)
                        if _la==CnsMRParser.Equ_op:
                            self.state = 518
                            self.match(CnsMRParser.Equ_op)


                        self.state = 521
                        self.selection()


                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 526
                self.match(CnsMRParser.R_paren)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sani_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def sani_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Sani_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_sani_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSani_statement" ):
                listener.enterSani_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSani_statement" ):
                listener.exitSani_statement(self)




    def sani_statement(self):

        localctx = CnsMRParser.Sani_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 42, self.RULE_sani_statement)
        self._la = 0 # Token type
        try:
            self.state = 560
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 530
                self.sani_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 531
                self.match(CnsMRParser.Classification)
                self.state = 533
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 532
                    self.match(CnsMRParser.Equ_op)


                self.state = 535
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 536
                self.match(CnsMRParser.Coefficients)
                self.state = 537
                self.number_s()
                self.state = 538
                self.number_s()
                self.state = 539
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 541
                self.match(CnsMRParser.ForceConstant)
                self.state = 543
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 542
                    self.match(CnsMRParser.Equ_op)


                self.state = 545
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 546
                self.match(CnsMRParser.Nrestraints)
                self.state = 548
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 547
                    self.match(CnsMRParser.Equ_op)


                self.state = 550
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 551
                self.match(CnsMRParser.Potential)
                self.state = 553
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 552
                    self.match(CnsMRParser.Equ_op)


                self.state = 555
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 556
                self.match(CnsMRParser.Print)
                self.state = 557
                self.match(CnsMRParser.Threshold)
                self.state = 558
                self.number_s()
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 559
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Sani_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_sani_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSani_assign" ):
                listener.enterSani_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSani_assign" ):
                listener.exitSani_assign(self)




    def sani_assign(self):

        localctx = CnsMRParser.Sani_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 44, self.RULE_sani_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 562
            self.match(CnsMRParser.Assign)
            self.state = 563
            self.selection()
            self.state = 564
            self.selection()
            self.state = 565
            self.selection()
            self.state = 566
            self.selection()
            self.state = 567
            self.selection()
            self.state = 568
            self.selection()
            self.state = 569
            self.number()
            self.state = 570
            self.number()
            self.state = 572
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Integer or _la==CnsMRParser.Real:
                self.state = 571
                self.number()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coupling_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def coup_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Coup_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def Cv(self):
            return self.getToken(CnsMRParser.Cv, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Partition(self):
            return self.getToken(CnsMRParser.Partition, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Coupling_potential(self):
            return self.getToken(CnsMRParser.Coupling_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_coupling_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoupling_statement" ):
                listener.enterCoupling_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoupling_statement" ):
                listener.exitCoupling_statement(self)




    def coupling_statement(self):

        localctx = CnsMRParser.Coupling_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 46, self.RULE_coupling_statement)
        self._la = 0 # Token type
        try:
            self.state = 623
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 574
                self.coup_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 575
                self.match(CnsMRParser.Classification)
                self.state = 577
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 576
                    self.match(CnsMRParser.Equ_op)


                self.state = 579
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 580
                self.match(CnsMRParser.Coefficients)
                self.state = 581
                self.number_s()
                self.state = 582
                self.number_s()
                self.state = 583
                self.number_s()
                self.state = 584
                self.number_s()
                pass
            elif token in [CnsMRParser.Cv]:
                self.enterOuterAlt(localctx, 4)
                self.state = 586
                self.match(CnsMRParser.Cv)
                self.state = 588
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 587
                    self.match(CnsMRParser.Equ_op)


                self.state = 590
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 5)
                self.state = 591
                self.match(CnsMRParser.ForceConstant)
                self.state = 592
                self.number_s()
                self.state = 594
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer or _la==CnsMRParser.Real:
                    self.state = 593
                    self.number_s()


                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 6)
                self.state = 596
                self.match(CnsMRParser.Nrestraints)
                self.state = 598
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 597
                    self.match(CnsMRParser.Equ_op)


                self.state = 600
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Partition]:
                self.enterOuterAlt(localctx, 7)
                self.state = 601
                self.match(CnsMRParser.Partition)
                self.state = 603
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 602
                    self.match(CnsMRParser.Equ_op)


                self.state = 605
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 8)
                self.state = 606
                self.match(CnsMRParser.Potential)
                self.state = 608
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 607
                    self.match(CnsMRParser.Equ_op)


                self.state = 610
                self.match(CnsMRParser.Coupling_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 9)
                self.state = 611
                self.match(CnsMRParser.Print)
                self.state = 612
                self.match(CnsMRParser.Threshold)
                self.state = 613
                self.number_s()
                self.state = 620
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 614
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 615
                    self.match(CnsMRParser.Classification)
                    self.state = 617
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 616
                        self.match(CnsMRParser.Equ_op)


                    self.state = 619
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 10)
                self.state = 622
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Coup_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_coup_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCoup_assign" ):
                listener.enterCoup_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCoup_assign" ):
                listener.exitCoup_assign(self)




    def coup_assign(self):

        localctx = CnsMRParser.Coup_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 48, self.RULE_coup_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 625
            self.match(CnsMRParser.Assign)
            self.state = 626
            self.selection()
            self.state = 627
            self.selection()
            self.state = 628
            self.selection()
            self.state = 629
            self.selection()
            self.state = 635
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 630
                self.selection()
                self.state = 631
                self.selection()
                self.state = 632
                self.selection()
                self.state = 633
                self.selection()


            self.state = 637
            self.number()
            self.state = 638
            self.number()
            self.state = 642
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Integer or _la==CnsMRParser.Real:
                self.state = 639
                self.number()
                self.state = 640
                self.number()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def carbon_shift_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Carbon_shift_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def PhiStep(self):
            return self.getToken(CnsMRParser.PhiStep, 0)

        def PsiStep(self):
            return self.getToken(CnsMRParser.PsiStep, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def carbon_shift_rcoil(self):
            return self.getTypedRuleContext(CnsMRParser.Carbon_shift_rcoilContext,0)


        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_statement" ):
                listener.enterCarbon_shift_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_statement" ):
                listener.exitCarbon_shift_statement(self)




    def carbon_shift_statement(self):

        localctx = CnsMRParser.Carbon_shift_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 50, self.RULE_carbon_shift_statement)
        self._la = 0 # Token type
        try:
            self.state = 688
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 644
                self.carbon_shift_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 645
                self.match(CnsMRParser.Classification)
                self.state = 647
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 646
                    self.match(CnsMRParser.Equ_op)


                self.state = 649
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Expectation]:
                self.enterOuterAlt(localctx, 3)
                self.state = 650
                self.match(CnsMRParser.Expectation)
                self.state = 651
                self.match(CnsMRParser.Integer)
                self.state = 652
                self.match(CnsMRParser.Integer)
                self.state = 653
                self.number_s()
                self.state = 654
                self.number_s()
                self.state = 655
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 657
                self.match(CnsMRParser.ForceConstant)
                self.state = 659
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 658
                    self.match(CnsMRParser.Equ_op)


                self.state = 661
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 662
                self.match(CnsMRParser.Nrestraints)
                self.state = 664
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 663
                    self.match(CnsMRParser.Equ_op)


                self.state = 666
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.PhiStep]:
                self.enterOuterAlt(localctx, 6)
                self.state = 667
                self.match(CnsMRParser.PhiStep)
                self.state = 669
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 668
                    self.match(CnsMRParser.Equ_op)


                self.state = 671
                self.number_s()
                pass
            elif token in [CnsMRParser.PsiStep]:
                self.enterOuterAlt(localctx, 7)
                self.state = 672
                self.match(CnsMRParser.PsiStep)
                self.state = 674
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 673
                    self.match(CnsMRParser.Equ_op)


                self.state = 676
                self.number_s()
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 8)
                self.state = 677
                self.match(CnsMRParser.Potential)
                self.state = 679
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 678
                    self.match(CnsMRParser.Equ_op)


                self.state = 681
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 9)
                self.state = 682
                self.match(CnsMRParser.Print)
                self.state = 683
                self.match(CnsMRParser.Threshold)
                self.state = 684
                self.number_s()
                pass
            elif token in [CnsMRParser.Rcoil]:
                self.enterOuterAlt(localctx, 10)
                self.state = 685
                self.carbon_shift_rcoil()
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 11)
                self.state = 686
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Zero]:
                self.enterOuterAlt(localctx, 12)
                self.state = 687
                self.match(CnsMRParser.Zero)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_assign" ):
                listener.enterCarbon_shift_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_assign" ):
                listener.exitCarbon_shift_assign(self)




    def carbon_shift_assign(self):

        localctx = CnsMRParser.Carbon_shift_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 52, self.RULE_carbon_shift_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 690
            self.match(CnsMRParser.Assign)
            self.state = 691
            self.selection()
            self.state = 692
            self.selection()
            self.state = 693
            self.selection()
            self.state = 694
            self.selection()
            self.state = 695
            self.selection()
            self.state = 696
            self.number()
            self.state = 697
            self.number()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Carbon_shift_rcoilContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Rcoil(self):
            return self.getToken(CnsMRParser.Rcoil, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_carbon_shift_rcoil

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterCarbon_shift_rcoil" ):
                listener.enterCarbon_shift_rcoil(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitCarbon_shift_rcoil" ):
                listener.exitCarbon_shift_rcoil(self)




    def carbon_shift_rcoil(self):

        localctx = CnsMRParser.Carbon_shift_rcoilContext(self, self._ctx, self.state)
        self.enterRule(localctx, 54, self.RULE_carbon_shift_rcoil)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 699
            self.match(CnsMRParser.Rcoil)
            self.state = 700
            self.selection()
            self.state = 701
            self.number_s()
            self.state = 702
            self.number_s()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def observed(self):
            return self.getTypedRuleContext(CnsMRParser.ObservedContext,0)


        def proton_shift_rcoil(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_rcoilContext,0)


        def proton_shift_anisotropy(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_anisotropyContext,0)


        def proton_shift_amides(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_amidesContext,0)


        def proton_shift_carbons(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_carbonsContext,0)


        def proton_shift_nitrogens(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_nitrogensContext,0)


        def proton_shift_oxygens(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_oxygensContext,0)


        def proton_shift_ring_atoms(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_ring_atomsContext,0)


        def proton_shift_alphas_and_amides(self):
            return self.getTypedRuleContext(CnsMRParser.Proton_shift_alphas_and_amidesContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Coupling_potential(self):
            return self.getToken(CnsMRParser.Coupling_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_statement" ):
                listener.enterProton_shift_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_statement" ):
                listener.exitProton_shift_statement(self)




    def proton_shift_statement(self):

        localctx = CnsMRParser.Proton_shift_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 56, self.RULE_proton_shift_statement)
        self._la = 0 # Token type
        try:
            self.state = 744
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Observed]:
                self.enterOuterAlt(localctx, 1)
                self.state = 704
                self.observed()
                pass
            elif token in [CnsMRParser.Rcoil]:
                self.enterOuterAlt(localctx, 2)
                self.state = 705
                self.proton_shift_rcoil()
                pass
            elif token in [CnsMRParser.Anisotropy]:
                self.enterOuterAlt(localctx, 3)
                self.state = 706
                self.proton_shift_anisotropy()
                pass
            elif token in [CnsMRParser.Amides]:
                self.enterOuterAlt(localctx, 4)
                self.state = 707
                self.proton_shift_amides()
                pass
            elif token in [CnsMRParser.Carbons]:
                self.enterOuterAlt(localctx, 5)
                self.state = 708
                self.proton_shift_carbons()
                pass
            elif token in [CnsMRParser.Nitrogens]:
                self.enterOuterAlt(localctx, 6)
                self.state = 709
                self.proton_shift_nitrogens()
                pass
            elif token in [CnsMRParser.Oxygens]:
                self.enterOuterAlt(localctx, 7)
                self.state = 710
                self.proton_shift_oxygens()
                pass
            elif token in [CnsMRParser.RingAtoms]:
                self.enterOuterAlt(localctx, 8)
                self.state = 711
                self.proton_shift_ring_atoms()
                pass
            elif token in [CnsMRParser.AlphasAndAmides]:
                self.enterOuterAlt(localctx, 9)
                self.state = 712
                self.proton_shift_alphas_and_amides()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 10)
                self.state = 713
                self.match(CnsMRParser.Classification)
                self.state = 715
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 714
                    self.match(CnsMRParser.Equ_op)


                self.state = 717
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Error]:
                self.enterOuterAlt(localctx, 11)
                self.state = 718
                self.match(CnsMRParser.Error)
                self.state = 720
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 719
                    self.match(CnsMRParser.Equ_op)


                self.state = 722
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 12)
                self.state = 723
                self.match(CnsMRParser.ForceConstant)
                self.state = 724
                self.number_s()
                self.state = 726
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer or _la==CnsMRParser.Real:
                    self.state = 725
                    self.number_s()


                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 13)
                self.state = 728
                self.match(CnsMRParser.Potential)
                self.state = 729
                self.match(CnsMRParser.Coupling_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 14)
                self.state = 730
                self.match(CnsMRParser.Print)
                self.state = 731
                self.match(CnsMRParser.Threshold)
                self.state = 732
                self.number_s()
                self.state = 739
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 733
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 734
                    self.match(CnsMRParser.Classification)
                    self.state = 736
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 735
                        self.match(CnsMRParser.Equ_op)


                    self.state = 738
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                self.state = 741
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 15)
                self.state = 743
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class ObservedContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Observed(self):
            return self.getToken(CnsMRParser.Observed, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_observed

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterObserved" ):
                listener.enterObserved(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitObserved" ):
                listener.exitObserved(self)




    def observed(self):

        localctx = CnsMRParser.ObservedContext(self, self._ctx, self.state)
        self.enterRule(localctx, 58, self.RULE_observed)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 746
            self.match(CnsMRParser.Observed)
            self.state = 747
            self.selection()
            self.state = 749
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 748
                self.selection()


            self.state = 751
            self.number_s()
            self.state = 753
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Integer or _la==CnsMRParser.Real:
                self.state = 752
                self.number_s()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_rcoilContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Rcoil(self):
            return self.getToken(CnsMRParser.Rcoil, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_rcoil

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_rcoil" ):
                listener.enterProton_shift_rcoil(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_rcoil" ):
                listener.exitProton_shift_rcoil(self)




    def proton_shift_rcoil(self):

        localctx = CnsMRParser.Proton_shift_rcoilContext(self, self._ctx, self.state)
        self.enterRule(localctx, 60, self.RULE_proton_shift_rcoil)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 755
            self.match(CnsMRParser.Rcoil)
            self.state = 756
            self.selection()
            self.state = 757
            self.number_s()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_anisotropyContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Anisotropy(self):
            return self.getToken(CnsMRParser.Anisotropy, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Logical(self):
            return self.getToken(CnsMRParser.Logical, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_anisotropy

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_anisotropy" ):
                listener.enterProton_shift_anisotropy(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_anisotropy" ):
                listener.exitProton_shift_anisotropy(self)




    def proton_shift_anisotropy(self):

        localctx = CnsMRParser.Proton_shift_anisotropyContext(self, self._ctx, self.state)
        self.enterRule(localctx, 62, self.RULE_proton_shift_anisotropy)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 759
            self.match(CnsMRParser.Anisotropy)
            self.state = 760
            self.selection()
            self.state = 761
            self.selection()
            self.state = 762
            self.selection()
            self.state = 763
            self.match(CnsMRParser.Simple_name)
            self.state = 765
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Logical:
                self.state = 764
                self.match(CnsMRParser.Logical)


            self.state = 767
            self.match(CnsMRParser.Simple_name)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_amidesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Amides(self):
            return self.getToken(CnsMRParser.Amides, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_amides

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_amides" ):
                listener.enterProton_shift_amides(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_amides" ):
                listener.exitProton_shift_amides(self)




    def proton_shift_amides(self):

        localctx = CnsMRParser.Proton_shift_amidesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 64, self.RULE_proton_shift_amides)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 769
            self.match(CnsMRParser.Amides)
            self.state = 770
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_carbonsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Carbons(self):
            return self.getToken(CnsMRParser.Carbons, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_carbons

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_carbons" ):
                listener.enterProton_shift_carbons(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_carbons" ):
                listener.exitProton_shift_carbons(self)




    def proton_shift_carbons(self):

        localctx = CnsMRParser.Proton_shift_carbonsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 66, self.RULE_proton_shift_carbons)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 772
            self.match(CnsMRParser.Carbons)
            self.state = 773
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_nitrogensContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Nitrogens(self):
            return self.getToken(CnsMRParser.Nitrogens, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_nitrogens

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_nitrogens" ):
                listener.enterProton_shift_nitrogens(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_nitrogens" ):
                listener.exitProton_shift_nitrogens(self)




    def proton_shift_nitrogens(self):

        localctx = CnsMRParser.Proton_shift_nitrogensContext(self, self._ctx, self.state)
        self.enterRule(localctx, 68, self.RULE_proton_shift_nitrogens)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 775
            self.match(CnsMRParser.Nitrogens)
            self.state = 776
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_oxygensContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Oxygens(self):
            return self.getToken(CnsMRParser.Oxygens, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_oxygens

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_oxygens" ):
                listener.enterProton_shift_oxygens(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_oxygens" ):
                listener.exitProton_shift_oxygens(self)




    def proton_shift_oxygens(self):

        localctx = CnsMRParser.Proton_shift_oxygensContext(self, self._ctx, self.state)
        self.enterRule(localctx, 70, self.RULE_proton_shift_oxygens)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 778
            self.match(CnsMRParser.Oxygens)
            self.state = 779
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_ring_atomsContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def RingAtoms(self):
            return self.getToken(CnsMRParser.RingAtoms, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_ring_atoms

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_ring_atoms" ):
                listener.enterProton_shift_ring_atoms(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_ring_atoms" ):
                listener.exitProton_shift_ring_atoms(self)




    def proton_shift_ring_atoms(self):

        localctx = CnsMRParser.Proton_shift_ring_atomsContext(self, self._ctx, self.state)
        self.enterRule(localctx, 72, self.RULE_proton_shift_ring_atoms)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 781
            self.match(CnsMRParser.RingAtoms)
            self.state = 782
            self.match(CnsMRParser.Simple_name)
            self.state = 783
            self.selection()
            self.state = 784
            self.selection()
            self.state = 785
            self.selection()
            self.state = 786
            self.selection()
            self.state = 787
            self.selection()
            self.state = 789
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 788
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Proton_shift_alphas_and_amidesContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def AlphasAndAmides(self):
            return self.getToken(CnsMRParser.AlphasAndAmides, 0)

        def selection(self):
            return self.getTypedRuleContext(CnsMRParser.SelectionContext,0)


        def getRuleIndex(self):
            return CnsMRParser.RULE_proton_shift_alphas_and_amides

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterProton_shift_alphas_and_amides" ):
                listener.enterProton_shift_alphas_and_amides(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitProton_shift_alphas_and_amides" ):
                listener.exitProton_shift_alphas_and_amides(self)




    def proton_shift_alphas_and_amides(self):

        localctx = CnsMRParser.Proton_shift_alphas_and_amidesContext(self, self._ctx, self.state)
        self.enterRule(localctx, 74, self.RULE_proton_shift_alphas_and_amides)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 791
            self.match(CnsMRParser.AlphasAndAmides)
            self.state = 792
            self.selection()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conformation_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def conf_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Conf_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Compressed(self):
            return self.getToken(CnsMRParser.Compressed, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Phase(self):
            return self.getToken(CnsMRParser.Phase, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Size(self):
            return self.getToken(CnsMRParser.Size, 0)

        def Dimensions(self):
            return self.getToken(CnsMRParser.Dimensions, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_conformation_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConformation_statement" ):
                listener.enterConformation_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConformation_statement" ):
                listener.exitConformation_statement(self)




    def conformation_statement(self):

        localctx = CnsMRParser.Conformation_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 76, self.RULE_conformation_statement)
        self._la = 0 # Token type
        try:
            self.state = 877
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 794
                self.conf_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 795
                self.match(CnsMRParser.Classification)
                self.state = 797
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 796
                    self.match(CnsMRParser.Equ_op)


                self.state = 799
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Compressed]:
                self.enterOuterAlt(localctx, 3)
                self.state = 800
                self.match(CnsMRParser.Compressed)
                pass
            elif token in [CnsMRParser.Expectation]:
                self.enterOuterAlt(localctx, 4)
                self.state = 801
                self.match(CnsMRParser.Expectation)
                self.state = 802
                self.match(CnsMRParser.Integer)
                self.state = 804
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,92,self._ctx)
                if la_ == 1:
                    self.state = 803
                    self.match(CnsMRParser.Integer)


                self.state = 807
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,93,self._ctx)
                if la_ == 1:
                    self.state = 806
                    self.match(CnsMRParser.Integer)


                self.state = 810
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,94,self._ctx)
                if la_ == 1:
                    self.state = 809
                    self.match(CnsMRParser.Integer)


                self.state = 812
                self.number_s()
                pass
            elif token in [CnsMRParser.Error]:
                self.enterOuterAlt(localctx, 5)
                self.state = 813
                self.match(CnsMRParser.Error)
                self.state = 815
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 814
                    self.match(CnsMRParser.Equ_op)


                self.state = 817
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 6)
                self.state = 818
                self.match(CnsMRParser.ForceConstant)
                self.state = 820
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 819
                    self.match(CnsMRParser.Equ_op)


                self.state = 822
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 7)
                self.state = 823
                self.match(CnsMRParser.Nrestraints)
                self.state = 825
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 824
                    self.match(CnsMRParser.Equ_op)


                self.state = 827
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Phase]:
                self.enterOuterAlt(localctx, 8)
                self.state = 828
                self.match(CnsMRParser.Phase)
                self.state = 829
                self.match(CnsMRParser.Integer)
                self.state = 830
                self.match(CnsMRParser.Integer)
                self.state = 831
                self.match(CnsMRParser.Integer)
                self.state = 835
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,98,self._ctx)
                if la_ == 1:
                    self.state = 832
                    self.match(CnsMRParser.Integer)
                    self.state = 833
                    self.match(CnsMRParser.Integer)
                    self.state = 834
                    self.match(CnsMRParser.Integer)


                self.state = 840
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,99,self._ctx)
                if la_ == 1:
                    self.state = 837
                    self.match(CnsMRParser.Integer)
                    self.state = 838
                    self.match(CnsMRParser.Integer)
                    self.state = 839
                    self.match(CnsMRParser.Integer)


                self.state = 845
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer:
                    self.state = 842
                    self.match(CnsMRParser.Integer)
                    self.state = 843
                    self.match(CnsMRParser.Integer)
                    self.state = 844
                    self.match(CnsMRParser.Integer)


                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 9)
                self.state = 847
                self.match(CnsMRParser.Potential)
                self.state = 849
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 848
                    self.match(CnsMRParser.Equ_op)


                self.state = 851
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 10)
                self.state = 852
                self.match(CnsMRParser.Print)
                self.state = 853
                self.match(CnsMRParser.Threshold)
                self.state = 854
                self.number_s()
                self.state = 861
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 855
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 856
                    self.match(CnsMRParser.Classification)
                    self.state = 858
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 857
                        self.match(CnsMRParser.Equ_op)


                    self.state = 860
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 11)
                self.state = 863
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Size]:
                self.enterOuterAlt(localctx, 12)
                self.state = 864
                self.match(CnsMRParser.Size)
                self.state = 865
                self.match(CnsMRParser.Dimensions)
                self.state = 866
                self.match(CnsMRParser.Integer)
                self.state = 868
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,104,self._ctx)
                if la_ == 1:
                    self.state = 867
                    self.match(CnsMRParser.Integer)


                self.state = 871
                self._errHandler.sync(self)
                la_ = self._interp.adaptivePredict(self._input,105,self._ctx)
                if la_ == 1:
                    self.state = 870
                    self.match(CnsMRParser.Integer)


                self.state = 874
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Integer:
                    self.state = 873
                    self.match(CnsMRParser.Integer)


                pass
            elif token in [CnsMRParser.Zero]:
                self.enterOuterAlt(localctx, 13)
                self.state = 876
                self.match(CnsMRParser.Zero)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Conf_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_conf_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterConf_assign" ):
                listener.enterConf_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitConf_assign" ):
                listener.exitConf_assign(self)




    def conf_assign(self):

        localctx = CnsMRParser.Conf_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 78, self.RULE_conf_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 879
            self.match(CnsMRParser.Assign)
            self.state = 880
            self.selection()
            self.state = 881
            self.selection()
            self.state = 882
            self.selection()
            self.state = 883
            self.selection()
            self.state = 889
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,108,self._ctx)
            if la_ == 1:
                self.state = 884
                self.selection()
                self.state = 885
                self.selection()
                self.state = 886
                self.selection()
                self.state = 887
                self.selection()


            self.state = 896
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,109,self._ctx)
            if la_ == 1:
                self.state = 891
                self.selection()
                self.state = 892
                self.selection()
                self.state = 893
                self.selection()
                self.state = 894
                self.selection()


            self.state = 903
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 898
                self.selection()
                self.state = 899
                self.selection()
                self.state = 900
                self.selection()
                self.state = 901
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Diffusion_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def dani_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Dani_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_diffusion_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDiffusion_statement" ):
                listener.enterDiffusion_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDiffusion_statement" ):
                listener.exitDiffusion_statement(self)




    def diffusion_statement(self):

        localctx = CnsMRParser.Diffusion_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 80, self.RULE_diffusion_statement)
        self._la = 0 # Token type
        try:
            self.state = 937
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 905
                self.dani_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 906
                self.match(CnsMRParser.Classification)
                self.state = 908
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 907
                    self.match(CnsMRParser.Equ_op)


                self.state = 910
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 911
                self.match(CnsMRParser.Coefficients)
                self.state = 912
                self.number_s()
                self.state = 913
                self.number_s()
                self.state = 914
                self.number_s()
                self.state = 915
                self.number_s()
                self.state = 916
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 918
                self.match(CnsMRParser.ForceConstant)
                self.state = 920
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 919
                    self.match(CnsMRParser.Equ_op)


                self.state = 922
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 923
                self.match(CnsMRParser.Nrestraints)
                self.state = 925
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 924
                    self.match(CnsMRParser.Equ_op)


                self.state = 927
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 928
                self.match(CnsMRParser.Potential)
                self.state = 930
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 929
                    self.match(CnsMRParser.Equ_op)


                self.state = 932
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 933
                self.match(CnsMRParser.Print)
                self.state = 934
                self.match(CnsMRParser.Threshold)
                self.state = 935
                self.number_s()
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 936
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Dani_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_dani_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterDani_assign" ):
                listener.enterDani_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitDani_assign" ):
                listener.exitDani_assign(self)




    def dani_assign(self):

        localctx = CnsMRParser.Dani_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 82, self.RULE_dani_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 939
            self.match(CnsMRParser.Assign)
            self.state = 940
            self.selection()
            self.state = 941
            self.selection()
            self.state = 942
            self.selection()
            self.state = 943
            self.selection()
            self.state = 944
            self.selection()
            self.state = 945
            self.selection()
            self.state = 946
            self.number()
            self.state = 947
            self.number()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_coupling_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def one_bond_assign(self):
            return self.getTypedRuleContext(CnsMRParser.One_bond_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def Coefficients(self):
            return self.getToken(CnsMRParser.Coefficients, 0)

        def number_s(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_sContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_sContext,i)


        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_coupling_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_coupling_statement" ):
                listener.enterOne_bond_coupling_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_coupling_statement" ):
                listener.exitOne_bond_coupling_statement(self)




    def one_bond_coupling_statement(self):

        localctx = CnsMRParser.One_bond_coupling_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 84, self.RULE_one_bond_coupling_statement)
        self._la = 0 # Token type
        try:
            self.state = 983
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 949
                self.one_bond_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 950
                self.match(CnsMRParser.Classification)
                self.state = 952
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 951
                    self.match(CnsMRParser.Equ_op)


                self.state = 954
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Coefficients]:
                self.enterOuterAlt(localctx, 3)
                self.state = 955
                self.match(CnsMRParser.Coefficients)
                self.state = 956
                self.number_s()
                self.state = 957
                self.number_s()
                self.state = 958
                self.number_s()
                self.state = 959
                self.number_s()
                self.state = 960
                self.number_s()
                self.state = 961
                self.number_s()
                self.state = 962
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 4)
                self.state = 964
                self.match(CnsMRParser.ForceConstant)
                self.state = 966
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 965
                    self.match(CnsMRParser.Equ_op)


                self.state = 968
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 5)
                self.state = 969
                self.match(CnsMRParser.Nrestraints)
                self.state = 971
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 970
                    self.match(CnsMRParser.Equ_op)


                self.state = 973
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 6)
                self.state = 974
                self.match(CnsMRParser.Potential)
                self.state = 976
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 975
                    self.match(CnsMRParser.Equ_op)


                self.state = 978
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 7)
                self.state = 979
                self.match(CnsMRParser.Print)
                self.state = 980
                self.match(CnsMRParser.Threshold)
                self.state = 981
                self.number_s()
                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 8)
                self.state = 982
                self.match(CnsMRParser.Reset)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class One_bond_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def number(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.NumberContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.NumberContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_one_bond_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterOne_bond_assign" ):
                listener.enterOne_bond_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitOne_bond_assign" ):
                listener.exitOne_bond_assign(self)




    def one_bond_assign(self):

        localctx = CnsMRParser.One_bond_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 86, self.RULE_one_bond_assign)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 985
            self.match(CnsMRParser.Assign)
            self.state = 986
            self.selection()
            self.state = 987
            self.selection()
            self.state = 988
            self.selection()
            self.state = 989
            self.selection()
            self.state = 990
            self.selection()
            self.state = 991
            self.selection()
            self.state = 992
            self.selection()
            self.state = 993
            self.selection()
            self.state = 994
            self.number()
            self.state = 995
            self.number()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def angle_db_assign(self):
            return self.getTypedRuleContext(CnsMRParser.Angle_db_assignContext,0)


        def Classification(self):
            return self.getToken(CnsMRParser.Classification, 0)

        def Simple_name(self):
            return self.getToken(CnsMRParser.Simple_name, 0)

        def Equ_op(self):
            return self.getToken(CnsMRParser.Equ_op, 0)

        def DerivFlag(self):
            return self.getToken(CnsMRParser.DerivFlag, 0)

        def Expectation(self):
            return self.getToken(CnsMRParser.Expectation, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def number_s(self):
            return self.getTypedRuleContext(CnsMRParser.Number_sContext,0)


        def Error(self):
            return self.getToken(CnsMRParser.Error, 0)

        def ForceConstant(self):
            return self.getToken(CnsMRParser.ForceConstant, 0)

        def Nrestraints(self):
            return self.getToken(CnsMRParser.Nrestraints, 0)

        def Potential(self):
            return self.getToken(CnsMRParser.Potential, 0)

        def Rdc_potential(self):
            return self.getToken(CnsMRParser.Rdc_potential, 0)

        def Print(self):
            return self.getToken(CnsMRParser.Print, 0)

        def Threshold(self):
            return self.getToken(CnsMRParser.Threshold, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Reset(self):
            return self.getToken(CnsMRParser.Reset, 0)

        def Size(self):
            return self.getToken(CnsMRParser.Size, 0)

        def Angle_dihedral(self):
            return self.getToken(CnsMRParser.Angle_dihedral, 0)

        def Zero(self):
            return self.getToken(CnsMRParser.Zero, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_statement" ):
                listener.enterAngle_db_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_statement" ):
                listener.exitAngle_db_statement(self)




    def angle_db_statement(self):

        localctx = CnsMRParser.Angle_db_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 88, self.RULE_angle_db_statement)
        self._la = 0 # Token type
        try:
            self.state = 1049
            self._errHandler.sync(self)
            token = self._input.LA(1)
            if token in [CnsMRParser.Assign]:
                self.enterOuterAlt(localctx, 1)
                self.state = 997
                self.angle_db_assign()
                pass
            elif token in [CnsMRParser.Classification]:
                self.enterOuterAlt(localctx, 2)
                self.state = 998
                self.match(CnsMRParser.Classification)
                self.state = 1000
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 999
                    self.match(CnsMRParser.Equ_op)


                self.state = 1002
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.DerivFlag]:
                self.enterOuterAlt(localctx, 3)
                self.state = 1003
                self.match(CnsMRParser.DerivFlag)
                self.state = 1005
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1004
                    self.match(CnsMRParser.Equ_op)


                self.state = 1007
                self.match(CnsMRParser.Simple_name)
                pass
            elif token in [CnsMRParser.Expectation]:
                self.enterOuterAlt(localctx, 4)
                self.state = 1008
                self.match(CnsMRParser.Expectation)
                self.state = 1009
                self.match(CnsMRParser.Integer)
                self.state = 1010
                self.match(CnsMRParser.Integer)
                self.state = 1011
                self.number_s()
                pass
            elif token in [CnsMRParser.Error]:
                self.enterOuterAlt(localctx, 5)
                self.state = 1012
                self.match(CnsMRParser.Error)
                self.state = 1014
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1013
                    self.match(CnsMRParser.Equ_op)


                self.state = 1016
                self.number_s()
                pass
            elif token in [CnsMRParser.ForceConstant]:
                self.enterOuterAlt(localctx, 6)
                self.state = 1017
                self.match(CnsMRParser.ForceConstant)
                self.state = 1019
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1018
                    self.match(CnsMRParser.Equ_op)


                self.state = 1021
                self.number_s()
                pass
            elif token in [CnsMRParser.Nrestraints]:
                self.enterOuterAlt(localctx, 7)
                self.state = 1022
                self.match(CnsMRParser.Nrestraints)
                self.state = 1024
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1023
                    self.match(CnsMRParser.Equ_op)


                self.state = 1026
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Potential]:
                self.enterOuterAlt(localctx, 8)
                self.state = 1027
                self.match(CnsMRParser.Potential)
                self.state = 1029
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1028
                    self.match(CnsMRParser.Equ_op)


                self.state = 1031
                self.match(CnsMRParser.Rdc_potential)
                pass
            elif token in [CnsMRParser.Print]:
                self.enterOuterAlt(localctx, 9)
                self.state = 1032
                self.match(CnsMRParser.Print)
                self.state = 1033
                self.match(CnsMRParser.Threshold)
                self.state = 1034
                self.number_s()
                self.state = 1041
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.All]:
                    self.state = 1035
                    self.match(CnsMRParser.All)
                    pass
                elif token in [CnsMRParser.Classification]:
                    self.state = 1036
                    self.match(CnsMRParser.Classification)
                    self.state = 1038
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 1037
                        self.match(CnsMRParser.Equ_op)


                    self.state = 1040
                    self.match(CnsMRParser.Simple_name)
                    pass
                else:
                    raise NoViableAltException(self)

                pass
            elif token in [CnsMRParser.Reset]:
                self.enterOuterAlt(localctx, 10)
                self.state = 1043
                self.match(CnsMRParser.Reset)
                pass
            elif token in [CnsMRParser.Size]:
                self.enterOuterAlt(localctx, 11)
                self.state = 1044
                self.match(CnsMRParser.Size)
                self.state = 1045
                self.match(CnsMRParser.Angle_dihedral)
                self.state = 1046
                self.match(CnsMRParser.Integer)
                self.state = 1047
                self.match(CnsMRParser.Integer)
                pass
            elif token in [CnsMRParser.Zero]:
                self.enterOuterAlt(localctx, 12)
                self.state = 1048
                self.match(CnsMRParser.Zero)
                pass
            else:
                raise NoViableAltException(self)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Angle_db_assignContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Assign(self):
            return self.getToken(CnsMRParser.Assign, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def getRuleIndex(self):
            return CnsMRParser.RULE_angle_db_assign

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterAngle_db_assign" ):
                listener.enterAngle_db_assign(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitAngle_db_assign" ):
                listener.exitAngle_db_assign(self)




    def angle_db_assign(self):

        localctx = CnsMRParser.Angle_db_assignContext(self, self._ctx, self.state)
        self.enterRule(localctx, 90, self.RULE_angle_db_assign)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1051
            self.match(CnsMRParser.Assign)
            self.state = 1052
            self.selection()
            self.state = 1053
            self.selection()
            self.state = 1054
            self.selection()
            self.state = 1055
            self.selection()
            self.state = 1056
            self.selection()
            self.state = 1057
            self.selection()
            self.state = 1058
            self.selection()
            self.state = 1059
            self.selection()
            self.state = 1060
            self.selection()
            self.state = 1061
            self.selection()
            self.state = 1062
            self.selection()
            self.state = 1064
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.L_paren:
                self.state = 1063
                self.selection()


        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class SelectionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def selection_expression(self):
            return self.getTypedRuleContext(CnsMRParser.Selection_expressionContext,0)


        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_selection

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSelection" ):
                listener.enterSelection(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSelection" ):
                listener.exitSelection(self)




    def selection(self):

        localctx = CnsMRParser.SelectionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 92, self.RULE_selection)
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1066
            self.match(CnsMRParser.L_paren)
            self.state = 1067
            self.selection_expression()
            self.state = 1068
            self.match(CnsMRParser.R_paren)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Selection_expressionContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def term(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.TermContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.TermContext,i)


        def Or_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Or_op)
            else:
                return self.getToken(CnsMRParser.Or_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_selection_expression

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterSelection_expression" ):
                listener.enterSelection_expression(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitSelection_expression" ):
                listener.exitSelection_expression(self)




    def selection_expression(self):

        localctx = CnsMRParser.Selection_expressionContext(self, self._ctx, self.state)
        self.enterRule(localctx, 94, self.RULE_selection_expression)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1070
            self.term()
            self.state = 1075
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Or_op:
                self.state = 1071
                self.match(CnsMRParser.Or_op)
                self.state = 1072
                self.term()
                self.state = 1077
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class TermContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def factor(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.FactorContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.FactorContext,i)


        def And_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.And_op)
            else:
                return self.getToken(CnsMRParser.And_op, i)

        def getRuleIndex(self):
            return CnsMRParser.RULE_term

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterTerm" ):
                listener.enterTerm(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitTerm" ):
                listener.exitTerm(self)




    def term(self):

        localctx = CnsMRParser.TermContext(self, self._ctx, self.state)
        self.enterRule(localctx, 96, self.RULE_term)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1078
            self.factor(0)
            self.state = 1083
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.And_op:
                self.state = 1079
                self.match(CnsMRParser.And_op)
                self.state = 1080
                self.factor(0)
                self.state = 1085
                self._errHandler.sync(self)
                _la = self._input.LA(1)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class FactorContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def L_paren(self):
            return self.getToken(CnsMRParser.L_paren, 0)

        def selection_expression(self):
            return self.getTypedRuleContext(CnsMRParser.Selection_expressionContext,0)


        def R_paren(self):
            return self.getToken(CnsMRParser.R_paren, 0)

        def All(self):
            return self.getToken(CnsMRParser.All, 0)

        def Atom(self):
            return self.getToken(CnsMRParser.Atom, 0)

        def Simple_names(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_names)
            else:
                return self.getToken(CnsMRParser.Simple_names, i)

        def Simple_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Simple_name)
            else:
                return self.getToken(CnsMRParser.Simple_name, i)

        def Integers(self):
            return self.getToken(CnsMRParser.Integers, 0)

        def Integer(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Integer)
            else:
                return self.getToken(CnsMRParser.Integer, i)

        def Attribute(self):
            return self.getToken(CnsMRParser.Attribute, 0)

        def Attr_properties(self):
            return self.getToken(CnsMRParser.Attr_properties, 0)

        def Comparison_ops(self):
            return self.getToken(CnsMRParser.Comparison_ops, 0)

        def number_f(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.Number_fContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.Number_fContext,i)


        def Abs(self):
            return self.getToken(CnsMRParser.Abs, 0)

        def BondedTo(self):
            return self.getToken(CnsMRParser.BondedTo, 0)

        def factor(self):
            return self.getTypedRuleContext(CnsMRParser.FactorContext,0)


        def ByGroup(self):
            return self.getToken(CnsMRParser.ByGroup, 0)

        def ByRes(self):
            return self.getToken(CnsMRParser.ByRes, 0)

        def Chemical(self):
            return self.getToken(CnsMRParser.Chemical, 0)

        def Colon(self):
            return self.getToken(CnsMRParser.Colon, 0)

        def Fbox(self):
            return self.getToken(CnsMRParser.Fbox, 0)

        def Hydrogen(self):
            return self.getToken(CnsMRParser.Hydrogen, 0)

        def Id(self):
            return self.getToken(CnsMRParser.Id, 0)

        def Known(self):
            return self.getToken(CnsMRParser.Known, 0)

        def Name(self):
            return self.getToken(CnsMRParser.Name, 0)

        def NONE(self):
            return self.getToken(CnsMRParser.NONE, 0)

        def Not_op(self):
            return self.getToken(CnsMRParser.Not_op, 0)

        def Point(self):
            return self.getToken(CnsMRParser.Point, 0)

        def Cut(self):
            return self.getToken(CnsMRParser.Cut, 0)

        def Comma(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Comma)
            else:
                return self.getToken(CnsMRParser.Comma, i)

        def Tail(self):
            return self.getToken(CnsMRParser.Tail, 0)

        def selection(self, i:int=None):
            if i is None:
                return self.getTypedRuleContexts(CnsMRParser.SelectionContext)
            else:
                return self.getTypedRuleContext(CnsMRParser.SelectionContext,i)


        def Equ_op(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Equ_op)
            else:
                return self.getToken(CnsMRParser.Equ_op, i)

        def Head(self):
            return self.getToken(CnsMRParser.Head, 0)

        def Previous(self):
            return self.getToken(CnsMRParser.Previous, 0)

        def Pseudo(self):
            return self.getToken(CnsMRParser.Pseudo, 0)

        def Residue(self):
            return self.getToken(CnsMRParser.Residue, 0)

        def Resname(self):
            return self.getToken(CnsMRParser.Resname, 0)

        def SegIdentifier(self):
            return self.getToken(CnsMRParser.SegIdentifier, 0)

        def Double_quote_string(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Double_quote_string)
            else:
                return self.getToken(CnsMRParser.Double_quote_string, i)

        def Sfbox(self):
            return self.getToken(CnsMRParser.Sfbox, 0)

        def Store_1(self):
            return self.getToken(CnsMRParser.Store_1, 0)

        def Store_2(self):
            return self.getToken(CnsMRParser.Store_2, 0)

        def Store_3(self):
            return self.getToken(CnsMRParser.Store_3, 0)

        def Store_4(self):
            return self.getToken(CnsMRParser.Store_4, 0)

        def Store_5(self):
            return self.getToken(CnsMRParser.Store_5, 0)

        def Store_6(self):
            return self.getToken(CnsMRParser.Store_6, 0)

        def Store_7(self):
            return self.getToken(CnsMRParser.Store_7, 0)

        def Store_8(self):
            return self.getToken(CnsMRParser.Store_8, 0)

        def Store_9(self):
            return self.getToken(CnsMRParser.Store_9, 0)

        def Tag(self):
            return self.getToken(CnsMRParser.Tag, 0)

        def Around(self):
            return self.getToken(CnsMRParser.Around, 0)

        def Saround(self):
            return self.getToken(CnsMRParser.Saround, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_factor

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFactor" ):
                listener.enterFactor(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFactor" ):
                listener.exitFactor(self)



    def factor(self, _p:int=0):
        _parentctx = self._ctx
        _parentState = self.state
        localctx = CnsMRParser.FactorContext(self, self._ctx, _parentState)
        _prevctx = localctx
        _startState = 98
        self.enterRecursionRule(localctx, 98, self.RULE_factor, _p)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1230
            self._errHandler.sync(self)
            la_ = self._interp.adaptivePredict(self._input,151,self._ctx)
            if la_ == 1:
                self.state = 1087
                self.match(CnsMRParser.L_paren)
                self.state = 1088
                self.selection_expression()
                self.state = 1089
                self.match(CnsMRParser.R_paren)
                pass

            elif la_ == 2:
                self.state = 1091
                self.match(CnsMRParser.All)
                pass

            elif la_ == 3:
                self.state = 1092
                self.match(CnsMRParser.Atom)
                self.state = 1093
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Simple_name or _la==CnsMRParser.Simple_names):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 1094
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Integers):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                self.state = 1095
                _la = self._input.LA(1)
                if not(_la==CnsMRParser.Simple_name or _la==CnsMRParser.Simple_names):
                    self._errHandler.recoverInline(self)
                else:
                    self._errHandler.reportMatch(self)
                    self.consume()
                pass

            elif la_ == 4:
                self.state = 1096
                self.match(CnsMRParser.Attribute)
                self.state = 1098
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Abs:
                    self.state = 1097
                    self.match(CnsMRParser.Abs)


                self.state = 1100
                self.match(CnsMRParser.Attr_properties)
                self.state = 1101
                self.match(CnsMRParser.Comparison_ops)
                self.state = 1102
                self.number_f()
                pass

            elif la_ == 5:
                self.state = 1103
                self.match(CnsMRParser.BondedTo)
                self.state = 1104
                self.factor(30)
                pass

            elif la_ == 6:
                self.state = 1105
                self.match(CnsMRParser.ByGroup)
                self.state = 1106
                self.factor(29)
                pass

            elif la_ == 7:
                self.state = 1107
                self.match(CnsMRParser.ByRes)
                self.state = 1108
                self.factor(28)
                pass

            elif la_ == 8:
                self.state = 1109
                self.match(CnsMRParser.Chemical)
                self.state = 1116
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1110
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1111
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1114
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,134,self._ctx)
                    if la_ == 1:
                        self.state = 1112
                        self.match(CnsMRParser.Colon)
                        self.state = 1113
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 9:
                self.state = 1118
                self.match(CnsMRParser.Fbox)
                self.state = 1119
                self.number_f()
                self.state = 1120
                self.number_f()
                self.state = 1121
                self.number_f()
                self.state = 1122
                self.number_f()
                self.state = 1123
                self.number_f()
                self.state = 1124
                self.number_f()
                pass

            elif la_ == 10:
                self.state = 1126
                self.match(CnsMRParser.Hydrogen)
                pass

            elif la_ == 11:
                self.state = 1127
                self.match(CnsMRParser.Id)
                self.state = 1128
                self.match(CnsMRParser.Integer)
                pass

            elif la_ == 12:
                self.state = 1129
                self.match(CnsMRParser.Known)
                pass

            elif la_ == 13:
                self.state = 1130
                self.match(CnsMRParser.Name)
                self.state = 1137
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1131
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1132
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1135
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,136,self._ctx)
                    if la_ == 1:
                        self.state = 1133
                        self.match(CnsMRParser.Colon)
                        self.state = 1134
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 14:
                self.state = 1139
                self.match(CnsMRParser.NONE)
                pass

            elif la_ == 15:
                self.state = 1140
                self.match(CnsMRParser.Not_op)
                self.state = 1141
                self.factor(20)
                pass

            elif la_ == 16:
                self.state = 1142
                self.match(CnsMRParser.Point)
                self.state = 1143
                self.match(CnsMRParser.L_paren)
                self.state = 1144
                self.number_f()
                self.state = 1146
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1145
                    self.match(CnsMRParser.Comma)


                self.state = 1148
                self.number_f()
                self.state = 1150
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1149
                    self.match(CnsMRParser.Comma)


                self.state = 1152
                self.number_f()
                self.state = 1153
                self.match(CnsMRParser.R_paren)
                self.state = 1154
                self.match(CnsMRParser.Cut)
                self.state = 1155
                self.number_f()
                pass

            elif la_ == 17:
                self.state = 1157
                self.match(CnsMRParser.Point)
                self.state = 1158
                self.match(CnsMRParser.L_paren)
                self.state = 1159
                self.match(CnsMRParser.Tail)
                self.state = 1161
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Equ_op:
                    self.state = 1160
                    self.match(CnsMRParser.Equ_op)


                self.state = 1163
                self.selection()
                self.state = 1165
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Comma:
                    self.state = 1164
                    self.match(CnsMRParser.Comma)


                self.state = 1172
                self._errHandler.sync(self)
                _la = self._input.LA(1)
                if _la==CnsMRParser.Head:
                    self.state = 1167
                    self.match(CnsMRParser.Head)
                    self.state = 1169
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    if _la==CnsMRParser.Equ_op:
                        self.state = 1168
                        self.match(CnsMRParser.Equ_op)


                    self.state = 1171
                    self.selection()


                self.state = 1174
                self.match(CnsMRParser.R_paren)
                self.state = 1175
                self.match(CnsMRParser.Cut)
                self.state = 1176
                self.number_f()
                pass

            elif la_ == 18:
                self.state = 1178
                self.match(CnsMRParser.Previous)
                pass

            elif la_ == 19:
                self.state = 1179
                self.match(CnsMRParser.Pseudo)
                pass

            elif la_ == 20:
                self.state = 1180
                self.match(CnsMRParser.Residue)
                self.state = 1187
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Integers]:
                    self.state = 1181
                    self.match(CnsMRParser.Integers)
                    pass
                elif token in [CnsMRParser.Integer]:
                    self.state = 1182
                    self.match(CnsMRParser.Integer)
                    self.state = 1185
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,144,self._ctx)
                    if la_ == 1:
                        self.state = 1183
                        self.match(CnsMRParser.Colon)
                        self.state = 1184
                        self.match(CnsMRParser.Integer)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 21:
                self.state = 1189
                self.match(CnsMRParser.Resname)
                self.state = 1196
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1190
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1191
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1194
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,146,self._ctx)
                    if la_ == 1:
                        self.state = 1192
                        self.match(CnsMRParser.Colon)
                        self.state = 1193
                        self.match(CnsMRParser.Simple_name)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 22:
                self.state = 1198
                self.match(CnsMRParser.SegIdentifier)
                self.state = 1210
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Simple_names]:
                    self.state = 1199
                    self.match(CnsMRParser.Simple_names)
                    pass
                elif token in [CnsMRParser.Simple_name]:
                    self.state = 1200
                    self.match(CnsMRParser.Simple_name)
                    self.state = 1203
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,148,self._ctx)
                    if la_ == 1:
                        self.state = 1201
                        self.match(CnsMRParser.Colon)
                        self.state = 1202
                        self.match(CnsMRParser.Simple_name)


                    pass
                elif token in [CnsMRParser.Double_quote_string]:
                    self.state = 1205
                    self.match(CnsMRParser.Double_quote_string)
                    self.state = 1208
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,149,self._ctx)
                    if la_ == 1:
                        self.state = 1206
                        self.match(CnsMRParser.Colon)
                        self.state = 1207
                        self.match(CnsMRParser.Double_quote_string)


                    pass
                else:
                    raise NoViableAltException(self)

                pass

            elif la_ == 23:
                self.state = 1212
                self.match(CnsMRParser.Sfbox)
                self.state = 1213
                self.number_f()
                self.state = 1214
                self.number_f()
                self.state = 1215
                self.number_f()
                self.state = 1216
                self.number_f()
                self.state = 1217
                self.number_f()
                self.state = 1218
                self.number_f()
                pass

            elif la_ == 24:
                self.state = 1220
                self.match(CnsMRParser.Store_1)
                pass

            elif la_ == 25:
                self.state = 1221
                self.match(CnsMRParser.Store_2)
                pass

            elif la_ == 26:
                self.state = 1222
                self.match(CnsMRParser.Store_3)
                pass

            elif la_ == 27:
                self.state = 1223
                self.match(CnsMRParser.Store_4)
                pass

            elif la_ == 28:
                self.state = 1224
                self.match(CnsMRParser.Store_5)
                pass

            elif la_ == 29:
                self.state = 1225
                self.match(CnsMRParser.Store_6)
                pass

            elif la_ == 30:
                self.state = 1226
                self.match(CnsMRParser.Store_7)
                pass

            elif la_ == 31:
                self.state = 1227
                self.match(CnsMRParser.Store_8)
                pass

            elif la_ == 32:
                self.state = 1228
                self.match(CnsMRParser.Store_9)
                pass

            elif la_ == 33:
                self.state = 1229
                self.match(CnsMRParser.Tag)
                pass


            self._ctx.stop = self._input.LT(-1)
            self.state = 1240
            self._errHandler.sync(self)
            _alt = self._interp.adaptivePredict(self._input,153,self._ctx)
            while _alt!=2 and _alt!=ATN.INVALID_ALT_NUMBER:
                if _alt==1:
                    if self._parseListeners is not None:
                        self.triggerExitRuleEvent()
                    _prevctx = localctx
                    self.state = 1238
                    self._errHandler.sync(self)
                    la_ = self._interp.adaptivePredict(self._input,152,self._ctx)
                    if la_ == 1:
                        localctx = CnsMRParser.FactorContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_factor)
                        self.state = 1232
                        if not self.precpred(self._ctx, 33):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 33)")
                        self.state = 1233
                        self.match(CnsMRParser.Around)
                        self.state = 1234
                        self.number_f()
                        pass

                    elif la_ == 2:
                        localctx = CnsMRParser.FactorContext(self, _parentctx, _parentState)
                        self.pushNewRecursionContext(localctx, _startState, self.RULE_factor)
                        self.state = 1235
                        if not self.precpred(self._ctx, 13):
                            from antlr4.error.Errors import FailedPredicateException
                            raise FailedPredicateException(self, "self.precpred(self._ctx, 13)")
                        self.state = 1236
                        self.match(CnsMRParser.Saround)
                        self.state = 1237
                        self.number_f()
                        pass

             
                self.state = 1242
                self._errHandler.sync(self)
                _alt = self._interp.adaptivePredict(self._input,153,self._ctx)

        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.unrollRecursionContexts(_parentctx)
        return localctx


    class NumberContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_number

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber" ):
                listener.enterNumber(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber" ):
                listener.exitNumber(self)




    def number(self):

        localctx = CnsMRParser.NumberContext(self, self._ctx, self.state)
        self.enterRule(localctx, 100, self.RULE_number)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1243
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Number_fContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_number_f

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber_f" ):
                listener.enterNumber_f(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber_f" ):
                listener.exitNumber_f(self)




    def number_f(self):

        localctx = CnsMRParser.Number_fContext(self, self._ctx, self.state)
        self.enterRule(localctx, 102, self.RULE_number_f)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1245
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Number_sContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Real(self):
            return self.getToken(CnsMRParser.Real, 0)

        def Integer(self):
            return self.getToken(CnsMRParser.Integer, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_number_s

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterNumber_s" ):
                listener.enterNumber_s(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitNumber_s" ):
                listener.exitNumber_s(self)




    def number_s(self):

        localctx = CnsMRParser.Number_sContext(self, self._ctx, self.state)
        self.enterRule(localctx, 104, self.RULE_number_s)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1247
            _la = self._input.LA(1)
            if not(_la==CnsMRParser.Integer or _la==CnsMRParser.Real):
                self._errHandler.recoverInline(self)
            else:
                self._errHandler.reportMatch(self)
                self.consume()
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx


    class Flag_statementContext(ParserRuleContext):

        def __init__(self, parser, parent:ParserRuleContext=None, invokingState:int=-1):
            super().__init__(parent, invokingState)
            self.parser = parser

        def Flags(self):
            return self.getToken(CnsMRParser.Flags, 0)

        def Include(self):
            return self.getToken(CnsMRParser.Include, 0)

        def End_flag(self):
            return self.getToken(CnsMRParser.End_flag, 0)

        def Exclude(self):
            return self.getToken(CnsMRParser.Exclude, 0)

        def Class_name(self, i:int=None):
            if i is None:
                return self.getTokens(CnsMRParser.Class_name)
            else:
                return self.getToken(CnsMRParser.Class_name, i)

        def Any_class(self):
            return self.getToken(CnsMRParser.Any_class, 0)

        def getRuleIndex(self):
            return CnsMRParser.RULE_flag_statement

        def enterRule(self, listener:ParseTreeListener):
            if hasattr( listener, "enterFlag_statement" ):
                listener.enterFlag_statement(self)

        def exitRule(self, listener:ParseTreeListener):
            if hasattr( listener, "exitFlag_statement" ):
                listener.exitFlag_statement(self)




    def flag_statement(self):

        localctx = CnsMRParser.Flag_statementContext(self, self._ctx, self.state)
        self.enterRule(localctx, 106, self.RULE_flag_statement)
        self._la = 0 # Token type
        try:
            self.enterOuterAlt(localctx, 1)
            self.state = 1249
            self.match(CnsMRParser.Flags)
            self.state = 1260
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            if _la==CnsMRParser.Exclude:
                self.state = 1250
                self.match(CnsMRParser.Exclude)
                self.state = 1258
                self._errHandler.sync(self)
                token = self._input.LA(1)
                if token in [CnsMRParser.Include, CnsMRParser.Class_name]:
                    self.state = 1254
                    self._errHandler.sync(self)
                    _la = self._input.LA(1)
                    while _la==CnsMRParser.Class_name:
                        self.state = 1251
                        self.match(CnsMRParser.Class_name)
                        self.state = 1256
                        self._errHandler.sync(self)
                        _la = self._input.LA(1)

                    pass
                elif token in [CnsMRParser.Any_class]:
                    self.state = 1257
                    self.match(CnsMRParser.Any_class)
                    pass
                else:
                    raise NoViableAltException(self)



            self.state = 1262
            self.match(CnsMRParser.Include)
            self.state = 1266
            self._errHandler.sync(self)
            _la = self._input.LA(1)
            while _la==CnsMRParser.Class_name:
                self.state = 1263
                self.match(CnsMRParser.Class_name)
                self.state = 1268
                self._errHandler.sync(self)
                _la = self._input.LA(1)

            self.state = 1269
            self.match(CnsMRParser.End_flag)
        except RecognitionException as re:
            localctx.exception = re
            self._errHandler.reportError(self, re)
            self._errHandler.recover(self, re)
        finally:
            self.exitRule()
        return localctx



    def sempred(self, localctx:RuleContext, ruleIndex:int, predIndex:int):
        if self._predicates == None:
            self._predicates = dict()
        self._predicates[49] = self.factor_sempred
        pred = self._predicates.get(ruleIndex, None)
        if pred is None:
            raise Exception("No predicate with index:" + str(ruleIndex))
        else:
            return pred(localctx, predIndex)

    def factor_sempred(self, localctx:FactorContext, predIndex:int):
            if predIndex == 0:
                return self.precpred(self._ctx, 33)
         

            if predIndex == 1:
                return self.precpred(self._ctx, 13)
         




